# -*- coding: utf-8 -*-
"""
SAP2000 LangChain Tools 模块

重构说明:
- 所有工具复用 PySap2000 已有封装，避免重复实现
- 工具只负责 @tool 装饰和 JSON 序列化
- 统一使用 base.py 中的响应函数

工具分类：
1. 模型信息 - 获取模型基本信息、组、截面等
2. 节点操作 - 创建、查询、删除节点
3. 杆件操作 - 创建、查询、修改杆件
4. 面单元操作 - 查询面单元信息
5. 荷载操作 - 添加、查询荷载
6. 分析设计 - 运行分析、钢结构设计
7. 结果查询 - 位移、内力、反力
8. 选择操作 - 选择、清除选择
9. 视图操作 - 刷新视图、缩放
10. 统计功能 - 用钢量、用索量
11. 批量操作 - 批量修改截面、荷载、约束
12. 绑图工具 - 通用图表绑制

Human-in-the-loop:
- QUERY_TOOLS: 查询类工具，自动执行
- MODIFY_TOOLS: 修改类工具，需要用户确认
"""

from .base import (
    QUERY_TOOLS,
    MODIFY_TOOLS,
    get_sap_model,
    safe_sap_call,
    SapConnectionError,
    SapModelError,
    to_json,
    success_response,
    error_response,
)

from .model_info import (
    check_connection,
    get_model_info,
    get_group_list,
    get_section_list,
    get_material_list,
    get_load_pattern_list,
    get_load_case_list,
    get_combo_list,
    get_section_info,
)

from .point import (
    get_point_coordinates,
    create_point,
    delete_point,
    get_point_restraint,
    set_point_restraint,
    get_point_list,
    get_points_in_group,
)

from .frame import (
    get_frame_info,
    create_frame,
    set_frame_section,
    delete_frame,
    get_frames_in_group,
    get_frame_list,
    set_frame_release,
    get_frame_release,
)

from .area import (
    create_area,
    delete_area,
    set_area_section,
    add_area_to_group_tool,
    get_area_info,
    get_areas_in_group,
    get_area_list,
)

from .cable_link import (
    create_cable,
    delete_cable,
    get_cable_info,
    get_cables_in_group,
    get_cable_list,
    create_link,
    delete_link,
    get_link_info,
    get_link_list,
    get_links_in_group,
)

from .load import (
    add_point_load,
    delete_point_load,
    add_frame_distributed_load,
    add_frame_point_load,
    delete_frame_load,
    get_frame_loads,
    add_area_load,
    delete_area_load,
    create_load_pattern,
    delete_load_pattern,
)

from .analysis import (
    run_analysis,
    run_steel_design,
    get_stress_ratios,
    verify_steel_design,
)

from .results import (
    get_point_displacement,
    get_point_reactions,
    get_base_reactions,
    get_frame_forces,
    get_max_frame_forces,
    get_modal_periods,
    get_modal_mass_ratios,
)

from .selection import (
    select_all,
    clear_selection,
    select_group,
    get_selected_objects,
    select_by_property,
)

from .view import (
    refresh_view,
    zoom_all,
)

from .statistics import (
    get_steel_usage,
    get_cable_usage,
)

from .group import (
    create_group,
    add_frame_to_group,
    add_selected_to_group,
)

from .file_ops import (
    save_model,
    unlock_model,
    new_model,
    open_model,
)

from .constraint import (
    get_constraint_list,
    create_diaphragm_constraint,
    assign_point_constraint,
)

from .edit import (
    move_selected_objects,
    replicate_linear,
    replicate_mirror,
    replicate_radial,
    divide_frame,
    merge_points,
)

from .table import (
    get_joint_coordinates_table,
    modify_joint_coordinate,
    batch_modify_joints,
    get_frame_section_assignments_table,
    get_available_tables,
)

from .history import (
    undo_last_operation,
    get_operation_history,
    clear_operation_history,
    add_to_history,
)

from .batch import (
    batch_set_section,
    batch_add_distributed_load,
    batch_set_restraint,
)

from .combo import (
    full_design_check,
    steel_usage_report,
    model_overview,
    COMBO_TOOLS,
)

# 可选模块（可能不存在）
try:
    from .rag_search import search_sap_docs
except ImportError:
    search_sap_docs = None

try:
    from .chart import draw_chart
except ImportError:
    draw_chart = None


def get_sap_tools():
    """返回所有 SAP2000 工具列表"""
    tools = [
        # 0. 连接检查
        check_connection,
        # 1. 模型信息
        get_model_info,
        get_group_list,
        get_section_list,
        get_material_list,
        get_load_pattern_list,
        get_load_case_list,
        get_combo_list,
        get_section_info,
        # 2. 节点操作
        get_point_coordinates,
        create_point,
        delete_point,
        get_point_restraint,
        set_point_restraint,
        get_point_list,
        get_points_in_group,
        # 3. 杆件操作
        get_frame_info,
        create_frame,
        set_frame_section,
        delete_frame,
        get_frames_in_group,
        get_frame_list,
        set_frame_release,
        get_frame_release,
        # 4. 面单元操作
        create_area,
        delete_area,
        set_area_section,
        add_area_to_group_tool,
        get_area_info,
        get_areas_in_group,
        get_area_list,
        # 5. 索/连接单元
        create_cable,
        delete_cable,
        get_cable_info,
        get_cables_in_group,
        get_cable_list,
        create_link,
        delete_link,
        get_link_info,
        get_link_list,
        get_links_in_group,
        # 6. 荷载操作
        add_point_load,
        delete_point_load,
        add_frame_distributed_load,
        add_frame_point_load,
        delete_frame_load,
        get_frame_loads,
        add_area_load,
        delete_area_load,
        create_load_pattern,
        delete_load_pattern,
        # 7. 分析设计
        run_analysis,
        run_steel_design,
        get_stress_ratios,
        verify_steel_design,
        # 8. 结果查询
        get_point_displacement,
        get_point_reactions,
        get_base_reactions,
        get_frame_forces,
        get_max_frame_forces,
        get_modal_periods,
        get_modal_mass_ratios,
        # 9. 选择操作
        select_all,
        clear_selection,
        select_group,
        select_by_property,
        get_selected_objects,
        # 10. 视图操作
        refresh_view,
        zoom_all,
        # 11. 统计功能
        get_steel_usage,
        get_cable_usage,
        # 12. 组操作
        create_group,
        add_frame_to_group,
        add_selected_to_group,
        # 13. 文件操作
        save_model,
        unlock_model,
        new_model,
        open_model,
        # 14. 约束操作
        get_constraint_list,
        create_diaphragm_constraint,
        assign_point_constraint,
        # 15. 编辑操作
        move_selected_objects,
        replicate_linear,
        replicate_mirror,
        replicate_radial,
        divide_frame,
        merge_points,
        # 16. 表格操作
        get_joint_coordinates_table,
        modify_joint_coordinate,
        batch_modify_joints,
        get_frame_section_assignments_table,
        get_available_tables,
        # 17. 撤回操作
        undo_last_operation,
        get_operation_history,
        clear_operation_history,
        # 18. 批量操作
        batch_set_section,
        batch_add_distributed_load,
        batch_set_restraint,
        # 19. 组合工具
        full_design_check,
        steel_usage_report,
        model_overview,
    ]
    
    # 添加可选工具
    if search_sap_docs:
        tools.append(search_sap_docs)
    if draw_chart:
        tools.append(draw_chart)
    
    return tools




"""
format_trans — 
Author: JIANGYAO-AISA
Date: 2024/2/9

"""
import re
class ConvertMyString:

    @staticmethod
    def convert_to_CHS_format(string):
        # 使用正则表达式提取数字部分
        numbers = re.findall(r'\d+', string)
        if len(numbers) >= 2:
            return f"CHS {numbers[0]}/{numbers[1]}/H"
        else:
            return "Invalid input"



    @staticmethod
    def convert_to_RHSU_format(input_str):
        """
        Convert rectangular section format from "B400x300x12x16" to "RHSU 400/300/16/16/12/12".

        Parameters:
        input_str (str): The input string in the format of "B<width>x<height>x<top_bottom_thickness>x<left_right_thickness>".

        Returns:
        str: The converted string in the format of "RHSU <width>/<height>/<left_right_thickness>/<left_right_thickness>/<top_bottom_thickness>/<top_bottom_thickness>".
        """
        # 检查输入字符串是否符合预期格式
        if not input_str.startswith('B') or 'x' not in input_str:
            raise ValueError(
                "Input string format is incorrect. Expected format 'B<width>x<height>x<top_bottom_thickness>x<left_right_thickness>'.")

        # 提取宽度、高度、上下壁厚、左右壁厚
        parts = input_str[1:].split('x')
        if len(parts) != 4:
            raise ValueError(
                "Input string format is incorrect. Expected format 'B<width>x<height>x<top_bottom_thickness>x<left_right_thickness>'.")

        width, height, top_bottom_thickness, left_right_thickness = parts

        # 构造并返回RHSU格式的字符串
        return f"RHSU {width}/{height}/{left_right_thickness}/{left_right_thickness}/{top_bottom_thickness}/{top_bottom_thickness}"


if __name__ == '__main__':
    f = ConvertMyString()
    input_str1 = "P219x8"
    converted_str1 = f.convert_to_CHS_format(input_str1)
    print(converted_str1)  # 输出: CHS 219/8/H
    input_str = "B400x300x12x16"
    converted_str = f.convert_to_RHSU_format(input_str)
    print(converted_str)  # 输出: RHSU 400/300/16/16/12/12




# -*- coding: utf-8 -*-
"""
分析软件互操作基类

所有分析软件导出器的共同特点：
- 只需要结构数据（节点、单元、截面属性）
- 不需要截面轮廓和3D网格
- 用于结构分析计算
"""

from abc import ABC, abstractmethod
from typing import Any, Dict, List


class AnalysisExporter(ABC):
    """分析软件导出器基类"""
    
    def __init__(self):
        """初始化分析软件导出器"""
        pass
    
    @abstractmethod
    def export(self, model_3d: Any, output_path: str) -> None:
        """
        导出模型为目标软件格式
        
        Args:
            model_3d: 3D模型数据
            output_path: 输出文件路径
        """
        pass
    
    def export_nodes(self, model_3d: Any) -> List[Dict]:
        """
        导出节点数据
        
        Args:
            model_3d: 3D模型数据
            
        Returns:
            节点列表 [{"id": 1, "x": 0, "y": 0, "z": 0}, ...]
        """
        nodes = []
        for point in model_3d.points:
            nodes.append({
                "id": point.name,
                "x": point.x,
                "y": point.y,
                "z": point.z
            })
        return nodes
    
    def export_elements(self, model_3d: Any) -> List[Dict]:
        """
        导出单元数据
        
        Args:
            model_3d: 3D模型数据
            
        Returns:
            单元列表 [{"id": 1, "nodes": [1, 2], "section": "W12X26"}, ...]
        """
        elements = []
        for elem in model_3d.elements:
            elements.append({
                "id": elem.name,
                "nodes": [elem.point_i, elem.point_j],
                "section": elem.section_name,
                "material": elem.material
            })
        return elements
    
    def export_section_properties(self, model_3d: Any) -> List[Dict]:
        """
        导出截面属性（不含轮廓）
        
        Args:
            model_3d: 3D模型数据
            
        Returns:
            截面属性列表 [{"name": "W12X26", "area": 0.01, "Ixx": 0.001, ...}, ...]
        """
        sections = []
        # 从 section_params 提取数值属性
        for elem in model_3d.elements:
            if elem.section_name not in [s["name"] for s in sections]:
                sections.append({
                    "name": elem.section_name,
                    "type": elem.section_type,
                    "params": elem.section_params  # 只有数值，不含轮廓
                })
        return sections
    
    def export_materials(self, model_3d: Any) -> List[Dict]:
        """
        导出材料数据
        
        Args:
            model_3d: 3D模型数据
            
        Returns:
            材料列表 [{"name": "STEEL", "E": 2e11, "nu": 0.3, ...}, ...]
        """
        materials = []
        # 提取唯一材料
        unique_materials = set(elem.material for elem in model_3d.elements)
        for mat in unique_materials:
            materials.append({
                "name": mat,
                # 材料属性需要从 SAP2000 API 获取
            })
        return materials
    
    def export_loads(self, model_3d: Any) -> List[Dict]:
        """
        导出荷载数据
        
        Args:
            model_3d: 3D模型数据
            
        Returns:
            荷载列表
        """
        # 待实现
        return []



# -*- coding: utf-8 -*-
"""
frame.py - 框架杆件数据对象
对应 SAP2000 的 FrameObj

这是一个纯数据类，只包含核心 CRUD 操作。
扩展功能（荷载、释放、修改器等）请使用:
- loads/frame_load.py - 荷载函数
- types_for_frames/ - 其他扩展函数

API Reference:
    - AddByCoord(xi, yi, zi, xj, yj, zj, Name, PropName, UserName, CSys) -> Long
    - AddByPoint(Point1, Point2, Name, PropName, UserName) -> Long
    - GetPoints(Name, Point1, Point2) -> Long
    - GetSection(Name, PropName, SAuto) -> Long
    - SetSection(Name, PropName, ItemType) -> Long
    - Count() -> Long
    - GetNameList() -> (NumberNames, MyName[], ret)
    - Delete(Name) -> Long

Usage:
    from PySap2000.structure_core import Frame
    
    # 通过节点创建杆件
    frame = Frame(no=1, start_point="1", end_point="2", section="W14X30")
    frame._create(model)
    
    # 通过坐标创建杆件
    frame = Frame(
        no=2, 
        start_x=0, start_y=0, start_z=0,
        end_x=10, end_y=0, end_z=0,
        section="W14X30"
    )
    frame._create(model)
    
    # 获取杆件
    frame = Frame.get_by_name(model, "1")
    print(f"截面: {frame.section}")
"""

import math
from dataclasses import dataclass, field
from typing import Optional, List, Tuple, Union, ClassVar

# 从 frame 模块导入枚举 (避免重复定义)
from frame.enums import (
    FrameType,
    FrameSectionType,
    FrameReleaseType,
    ItemType,
    SECTION_TYPE_NAMES,
)


# 截面类型到获取方法的映射
SECTION_TYPE_METHOD_MAP = {
    FrameSectionType.I_SECTION: 'GetISection_1',
    FrameSectionType.PIPE: 'GetPipe',
    FrameSectionType.BOX: 'GetTube_1',
    FrameSectionType.CIRCLE: 'GetCircle',
    FrameSectionType.RECTANGULAR: 'GetRectangle',
}


@dataclass
class Frame:
    """
    框架杆件数据对象
    
    对应 SAP2000 的 FrameObj
    
    Attributes:
        no: 杆件编号/名称
        start_point: 起始节点编号 (I-End)
        end_point: 结束节点编号 (J-End)
        section: 截面名称
        s_auto: 自动选择列表名称
        material: 材料名称
    """
    
    # 必填属性
    no: Union[int, str] = None
    
    # 通过节点定义
    start_point: Optional[Union[int, str]] = None
    end_point: Optional[Union[int, str]] = None
    
    # 通过坐标定义
    start_x: Optional[float] = None
    start_y: Optional[float] = None
    start_z: Optional[float] = None
    end_x: Optional[float] = None
    end_y: Optional[float] = None
    end_z: Optional[float] = None
    
    # 截面和材料
    section: str = ""
    s_auto: str = ""
    section_type: Optional[FrameSectionType] = None
    section_type_name: str = ""
    material: Optional[str] = None
    
    # 杆件属性
    type: FrameType = FrameType.BEAM
    local_axis_angle: float = 0.0
    advanced_axes: bool = False
    
    # 端部释放 (U1, U2, U3, R1, R2, R3)
    release_i: Tuple[bool, ...] = field(default_factory=lambda: (False,)*6)
    release_j: Tuple[bool, ...] = field(default_factory=lambda: (False,)*6)
    
    # 只读属性
    length: Optional[float] = field(default=None, repr=False)
    weight: float = 0.0  # 杆件重量 [kg]
    guid: Optional[str] = None
    
    # 可选属性
    coordinate_system: str = "Global"
    comment: str = ""
    
    # 类属性
    _object_type: ClassVar[str] = "FrameObj"
    
    # ==================== 核心 CRUD 方法 ====================
    
    def _create(self, model) -> int:
        """
        在 SAP2000 中创建杆件
        
        Returns:
            0 表示成功
        """
        user_name = str(self.no) if self.no is not None else ""
        section = self.section if self.section else "Default"
        
        # 通过坐标创建
        if self.start_x is not None and self.end_x is not None:
            result = model.FrameObj.AddByCoord(
                self.start_x, self.start_y or 0, self.start_z or 0,
                self.end_x, self.end_y or 0, self.end_z or 0,
                "", section, user_name, self.coordinate_system
            )
            if isinstance(result, (list, tuple)) and len(result) >= 2:
                if result[0]:
                    self.no = result[0]
                return result[-1]
            return result
        
        # 通过节点创建
        if self.start_point is not None and self.end_point is not None:
            result = model.FrameObj.AddByPoint(
                str(self.start_point), str(self.end_point),
                "", section, user_name
            )
            if isinstance(result, (list, tuple)) and len(result) >= 2:
                if result[0]:
                    self.no = result[0]
                return result[-1]
            return result
        
        from PySap2000.exceptions import FrameError
        raise FrameError("创建杆件需要指定节点或坐标")
    
    def _get(self, model) -> 'Frame':
        """从 SAP2000 获取杆件数据"""
        no_str = str(self.no)
        
        # 获取端点
        result = model.FrameObj.GetPoints(no_str)
        if isinstance(result, (list, tuple)) and len(result) >= 3:
            self.start_point = result[0]
            self.end_point = result[1]
            if result[2] != 0:
                from PySap2000.exceptions import FrameError
                raise FrameError(f"获取杆件 {no_str} 端点失败")
        
        # 获取截面
        result = model.FrameObj.GetSection(no_str)
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            self.section = result[0]
            self.s_auto = result[1] if len(result) > 1 and result[1] else ""
        
        # 获取截面类型
        self._get_section_type(model)
        
        # 获取局部轴
        self._get_local_axes(model)
        
        # 获取端部释放
        self._get_releases(model)
        
        # 获取 GUID
        self._get_guid(model)
        
        # 计算长度
        self._calculate_length(model)
        
        # 计算重量
        self._calculate_weight(model)
        
        return self
    
    def _delete(self, model) -> int:
        """从 SAP2000 删除杆件"""
        return model.FrameObj.Delete(str(self.no))
    
    def _update(self, model) -> int:
        """更新杆件截面"""
        if self.section:
            return model.FrameObj.SetSection(str(self.no), self.section, ItemType.OBJECT)
        return 0
    
    # ==================== 内部辅助方法 ====================
    
    def _get_section_type(self, model):
        """获取截面类型"""
        if self.section:
            try:
                result = model.PropFrame.GetTypeOAPI(self.section)
                if isinstance(result, (list, tuple)) and len(result) >= 1:
                    self.section_type = FrameSectionType(result[0])
                    self.section_type_name = SECTION_TYPE_NAMES.get(
                        self.section_type, self.section_type.name
                    )
            except (ValueError, Exception):
                pass
    
    def _get_local_axes(self, model):
        """获取局部轴角度"""
        try:
            result = model.FrameObj.GetLocalAxes(str(self.no))
            if isinstance(result, (list, tuple)) and len(result) >= 2:
                self.local_axis_angle = result[0]
                self.advanced_axes = result[1] if len(result) > 1 else False
        except Exception:
            pass
    
    def _get_releases(self, model):
        """获取端部释放"""
        try:
            result = model.FrameObj.GetReleases(str(self.no))
            if isinstance(result, (list, tuple)) and len(result) >= 2:
                self.release_i = tuple(result[0])
                self.release_j = tuple(result[1])
        except Exception:
            pass
    
    def _get_guid(self, model):
        """获取 GUID"""
        try:
            result = model.FrameObj.GetGUID(str(self.no))
            if isinstance(result, (list, tuple)) and len(result) >= 1:
                self.guid = result[0]
        except Exception:
            pass
    
    def _calculate_length(self, model):
        """计算杆件长度"""
        try:
            from PySap2000.structure_core.point import Point
            p1 = Point(no=self.start_point)._get(model)
            p2 = Point(no=self.end_point)._get(model)
            self.length = round(
                math.sqrt((p2.x - p1.x)**2 + (p2.y - p1.y)**2 + (p2.z - p1.z)**2),
                3
            )
        except Exception:
            self.length = None
    
    def _calculate_weight(self, model) -> float:
        """
        计算杆件重量 (kg)
        
        weight = weight_per_meter × length
        
        如果当前不是 N-m-C 单位，会临时切换获取数据
        
        Args:
            model: SapModel 对象
            
        Returns:
            杆件重量 (kg)，如果截面或长度无效则返回 0.0
        """
        if not self.section:
            self.weight = 0.0
            return 0.0
        
        try:
            from section.frame_section import FrameSection
            from global_parameters.units import Units, UnitSystem
            
            current_units = Units.get_present_units(model)
            need_switch = current_units != UnitSystem.N_M_C
            
            if need_switch:
                Units.set_present_units(model, UnitSystem.N_M_C)
            
            try:
                # 获取截面的单位长度重量 (kg/m)
                section = FrameSection.get_by_name(model, self.section)
                weight_per_meter = section.weight_per_meter
                
                if weight_per_meter <= 0:
                    self.weight = 0.0
                    return 0.0
                
                # 计算长度 (m)
                from PySap2000.structure_core.point import Point
                p1 = Point(no=self.start_point)._get(model)
                p2 = Point(no=self.end_point)._get(model)
                length_m = math.sqrt(
                    (p2.x - p1.x)**2 + (p2.y - p1.y)**2 + (p2.z - p1.z)**2
                )
                
                self.weight = weight_per_meter * length_m
            finally:
                if need_switch:
                    Units.set_present_units(model, current_units)
            
        except Exception:
            self.weight = 0.0
        
        return self.weight
    
    # ==================== 静态方法 ====================
    
    @staticmethod
    def get_count(model) -> int:
        """获取杆件总数"""
        return model.FrameObj.Count()
    
    @staticmethod
    def get_name_list(model) -> List[str]:
        """获取所有杆件名称列表"""
        result = model.FrameObj.GetNameList(0, [])
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            names = result[1]
            if names is not None:
                return list(names)
        return []
    
    @staticmethod
    def get_section_name_list(model) -> List[str]:
        """获取所有截面名称列表"""
        result = model.PropFrame.GetNameList(0, [])
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            names = result[1]
            if names is not None:
                return list(names)
        return []
    
    # ==================== 类方法 ====================
    
    @classmethod
    def get_by_name(cls, model, name: str) -> 'Frame':
        """
        获取指定名称的杆件
        
        Example:
            frame = Frame.get_by_name(model, "1")
            print(f"截面: {frame.section}")
        """
        frame = cls(no=name)
        frame._get(model)
        return frame
    
    @classmethod
    def get_all(cls, model, names: List[str] = None) -> List['Frame']:
        """
        获取所有杆件
        
        Example:
            frames = Frame.get_all(model)
            for f in frames:
                print(f"{f.no}: {f.section}")
        """
        if names is None:
            names = cls.get_name_list(model)
        return [cls.get_by_name(model, name) for name in names]
    
    # ==================== 批量操作方法 ====================
    
    @classmethod
    def calculate_weights_batch(
        cls, 
        model, 
        frames: List['Frame'] = None
    ) -> dict:
        """
        批量计算杆件重量，只切换一次单位
        
        相比逐个调用 _calculate_weight()，性能更好
        
        Args:
            model: SapModel 对象
            frames: 杆件列表，如果为 None 则获取所有杆件
            
        Returns:
            dict: {杆件名称: 重量(kg)}
            
        Example:
            weights = Frame.calculate_weights_batch(model)
            total = sum(weights.values())
            print(f"总重量: {total:.2f} kg")
        """
        from global_parameters.units import Units, UnitSystem
        from section.frame_section import FrameSection
        from PySap2000.structure_core.point import Point
        
        if frames is None:
            frames = cls.get_all(model)
        
        if not frames:
            return {}
        
        # 保存当前单位
        current_units = Units.get_present_units(model)
        need_switch = current_units != UnitSystem.N_M_C
        
        if need_switch:
            Units.set_present_units(model, UnitSystem.N_M_C)
        
        weights = {}
        section_cache = {}  # 缓存截面数据
        point_cache = {}    # 缓存节点坐标
        
        try:
            for frame in frames:
                try:
                    # 获取截面重量（带缓存）
                    if frame.section not in section_cache:
                        section = FrameSection.get_by_name(model, frame.section)
                        section_cache[frame.section] = section.weight_per_meter
                    
                    weight_per_meter = section_cache[frame.section]
                    if weight_per_meter <= 0:
                        weights[str(frame.no)] = 0.0
                        continue
                    
                    # 获取节点坐标（带缓存）
                    for pt_name in [frame.start_point, frame.end_point]:
                        if pt_name not in point_cache:
                            pt = Point(no=pt_name)._get(model)
                            point_cache[pt_name] = (pt.x, pt.y, pt.z)
                    
                    p1 = point_cache[frame.start_point]
                    p2 = point_cache[frame.end_point]
                    
                    # 计算长度和重量
                    length = math.sqrt(
                        (p2[0] - p1[0])**2 + 
                        (p2[1] - p1[1])**2 + 
                        (p2[2] - p1[2])**2
                    )
                    weights[str(frame.no)] = weight_per_meter * length
                    
                except Exception:
                    weights[str(frame.no)] = 0.0
        finally:
            if need_switch:
                Units.set_present_units(model, current_units)
        
        return weights
    
    @classmethod
    def create_batch(
        cls, 
        model, 
        frames: List['Frame']
    ) -> Tuple[List['Frame'], List[Tuple[str, str]]]:
        """
        批量创建杆件
        
        Args:
            model: SapModel 对象
            frames: 待创建的杆件列表
            
        Returns:
            Tuple[成功列表, 失败列表(名称, 错误信息)]
            
        Example:
            frames = [
                Frame(no="F1", start_point="1", end_point="2", section="W14X30"),
                Frame(no="F2", start_point="2", end_point="3", section="W14X30"),
            ]
            succeeded, failed = Frame.create_batch(model, frames)
            print(f"成功: {len(succeeded)}, 失败: {len(failed)}")
        """
        succeeded = []
        failed = []
        
        for frame in frames:
            try:
                ret = frame._create(model)
                if ret == 0:
                    succeeded.append(frame)
                else:
                    failed.append((str(frame.no), f"返回码: {ret}"))
            except Exception as e:
                failed.append((str(frame.no), str(e)))
        
        return succeeded, failed
    
    @classmethod
    def delete_batch(
        cls, 
        model, 
        names: List[str]
    ) -> Tuple[List[str], List[Tuple[str, str]]]:
        """
        批量删除杆件
        
        Args:
            model: SapModel 对象
            names: 待删除的杆件名称列表
            
        Returns:
            Tuple[成功名称列表, 失败列表(名称, 错误信息)]
            
        Example:
            succeeded, failed = Frame.delete_batch(model, ["F1", "F2", "F3"])
        """
        succeeded = []
        failed = []
        
        for name in names:
            try:
                ret = model.FrameObj.Delete(str(name))
                if ret == 0:
                    succeeded.append(name)
                else:
                    failed.append((name, f"返回码: {ret}"))
            except Exception as e:
                failed.append((name, str(e)))
        
        return succeeded, failed
    
    @classmethod
    def set_section_batch(
        cls, 
        model, 
        names: List[str], 
        section: str
    ) -> Tuple[List[str], List[Tuple[str, str]]]:
        """
        批量设置杆件截面
        
        Args:
            model: SapModel 对象
            names: 杆件名称列表
            section: 截面名称
            
        Returns:
            Tuple[成功名称列表, 失败列表(名称, 错误信息)]
            
        Example:
            succeeded, failed = Frame.set_section_batch(
                model, 
                ["F1", "F2", "F3"], 
                "W21X44"
            )
        """
        succeeded = []
        failed = []
        
        for name in names:
            try:
                ret = model.FrameObj.SetSection(str(name), section, ItemType.OBJECT)
                if ret == 0:
                    succeeded.append(name)
                else:
                    failed.append((name, f"返回码: {ret}"))
            except Exception as e:
                failed.append((name, str(e)))
        
        return succeeded, failed



# -*- coding: utf-8 -*-
"""
base_react.py - 基底反力结果函数

SAP2000 Results API 的基底反力函数封装

SAP2000 API:
- Results.BaseReact - 基底反力
"""

from typing import List
from .data_classes import BaseReactResult


def get_base_react(model) -> List[BaseReactResult]:
    """
    获取结构基底反力
    
    返回所有选中工况/组合的基底总反力。
    
    Args:
        model: SapModel 对象
        
    Returns:
        BaseReactResult 列表
        
    Example:
        from results import deselect_all_cases_and_combos, set_case_selected_for_output
        
        deselect_all_cases_and_combos(model)
        set_case_selected_for_output(model, "DEAD")
        
        results = get_base_react(model)
        for r in results:
            print(f"{r.load_case}: Fx={r.fx}, Fy={r.fy}, Fz={r.fz}")
    """
    result = model.Results.BaseReact(
        0, [], [], [],
        [], [], [], [], [], [],
        0.0, 0.0, 0.0
    )
    
    if isinstance(result, (list, tuple)) and len(result) >= 13:
        num = result[0]
        load_case = result[1]
        step_type = result[2]
        step_num = result[3]
        fx = result[4]
        fy = result[5]
        fz = result[6]
        mx = result[7]
        my = result[8]
        mz = result[9]
        gx = result[10]
        gy = result[11]
        gz = result[12]
        ret = result[-1] if len(result) > 13 else 0
        
        if (ret == 0 or len(result) == 13) and num > 0:
            return [
                BaseReactResult(
                    load_case=load_case[i] if load_case else "",
                    step_type=step_type[i] if step_type else "",
                    step_num=step_num[i] if step_num else 0.0,
                    fx=fx[i] if fx else 0.0,
                    fy=fy[i] if fy else 0.0,
                    fz=fz[i] if fz else 0.0,
                    mx=mx[i] if mx else 0.0,
                    my=my[i] if my else 0.0,
                    mz=mz[i] if mz else 0.0,
                    gx=gx if isinstance(gx, (int, float)) else 0.0,
                    gy=gy if isinstance(gy, (int, float)) else 0.0,
                    gz=gz if isinstance(gz, (int, float)) else 0.0,
                )
                for i in range(num)
            ]
    return []



# -*- coding: utf-8 -*-
"""
cad_export.py - CAD 出图模块

功能:
    - 导出杆件表格 (单元号、节点1、节点2、长度、材料、截面)
    - 导出节点表格 (节点号、X、Y、Z)
    - 支持导出选中对象或全部对象
    - 输出 DXF 文件 (可用 CAD 打开)
    - 支持使用模板文件

Usage:
    from application import Application
    from export_docs.cad_export import export_frame_table, export_point_table
    
    app = Application()
    model = app.model
    
    # 导出选中的杆件
    export_frame_table(model, "frames.dxf", selected_only=True)
    
    # 导出所有节点
    export_point_table(model, "points.dxf")
"""

from dataclasses import dataclass
from typing import List, Dict, Optional, Tuple
import math
import os


@dataclass
class FrameData:
    """杆件数据"""
    name: str
    point_i: str
    point_j: str
    length: float
    material: str
    section: str


@dataclass
class PointData:
    """节点数据"""
    name: str
    x: float
    y: float
    z: float


def get_selected_objects(model) -> Tuple[List[str], List[str]]:
    """
    获取选中的对象
    
    Returns:
        (frame_names, point_names)
    """
    ret = model.SelectObj.GetSelected(0, [], [])
    if not isinstance(ret, (list, tuple)) or len(ret) < 3:
        return [], []
    
    obj_types = ret[1] if ret[1] else []
    obj_names = ret[2] if ret[2] else []
    
    frames = []
    points = []
    for i, obj_type in enumerate(obj_types):
        if obj_type == 2:  # Frame
            frames.append(obj_names[i])
        elif obj_type == 1:  # Point
            points.append(obj_names[i])
    
    return frames, points


def get_frame_data(model, frame_names: List[str] = None) -> List[FrameData]:
    """
    获取杆件数据 (使用 DatabaseTables 批量获取，速度更快)
    
    Args:
        model: SapModel 对象
        frame_names: 杆件名称列表，None 表示全部
    """
    from global_parameters.units import Units, UnitSystem
    
    # 切换到 N-mm-C 单位
    original_units = Units.get_present_units(model)
    Units.set_present_units(model, UnitSystem.N_MM_C)
    
    try:
        # 1. 获取杆件连接信息 (Connectivity - Frame)
        conn_dict = {}  # frame -> (point_i, point_j, length)
        ret = model.DatabaseTables.GetTableForDisplayArray(
            "Connectivity - Frame", ["Frame", "JointI", "JointJ", "Length"], "", 0, [], 0, []
        )
        if isinstance(ret, (list, tuple)) and len(ret) >= 5 and ret[5] == 0:
            fields = list(ret[2])
            num_records = ret[3]
            data = ret[4]
            num_fields = len(fields)
            
            frame_idx = fields.index("Frame") if "Frame" in fields else -1
            ji_idx = fields.index("JointI") if "JointI" in fields else -1
            jj_idx = fields.index("JointJ") if "JointJ" in fields else -1
            len_idx = fields.index("Length") if "Length" in fields else -1
            
            for i in range(num_records):
                base = i * num_fields
                fname = data[base + frame_idx] if frame_idx >= 0 else ""
                if fname:
                    conn_dict[fname] = (
                        data[base + ji_idx] if ji_idx >= 0 else "",
                        data[base + jj_idx] if jj_idx >= 0 else "",
                        float(data[base + len_idx]) if len_idx >= 0 and data[base + len_idx] else 0.0
                    )
        
        if not conn_dict:
            return []
        
        # 2. 获取截面分配和材料覆盖 (Frame Section Assignments)
        section_dict = {}  # frame -> section
        mat_overwrite_dict = {}  # frame -> material (非Default时)
        ret = model.DatabaseTables.GetTableForDisplayArray(
            "Frame Section Assignments", ["Frame", "AnalSect", "MatProp"], "", 0, [], 0, []
        )
        if isinstance(ret, (list, tuple)) and len(ret) >= 5 and ret[5] == 0:
            fields = list(ret[2])
            num_records = ret[3]
            data = ret[4]
            num_fields = len(fields)
            
            frame_idx = fields.index("Frame") if "Frame" in fields else -1
            sect_idx = fields.index("AnalSect") if "AnalSect" in fields else -1
            mat_idx = fields.index("MatProp") if "MatProp" in fields else -1
            
            for i in range(num_records):
                base = i * num_fields
                fname = data[base + frame_idx] if frame_idx >= 0 else ""
                if fname:
                    section = data[base + sect_idx] if sect_idx >= 0 else ""
                    if section:
                        section_dict[fname] = section
                    # 获取材料覆盖 (非 Default 时有效)
                    if mat_idx >= 0:
                        mat = data[base + mat_idx]
                        if mat and mat != "Default":
                            mat_overwrite_dict[fname] = mat
        
        # 3. 缓存截面材料和标准名称
        section_material_cache = {}
        section_name_cache = {}  # 截面标准化名称缓存
        
        # 构建结果
        result = []
        frame_set = set(frame_names) if frame_names else None
        
        for fname, (point_i, point_j, length) in conn_dict.items():
            # 如果指定了杆件列表，只处理列表中的杆件
            if frame_set and fname not in frame_set:
                continue
            
            section = section_dict.get(fname, "")
            
            # 获取材料: 优先使用覆盖材料，否则从截面获取
            material = mat_overwrite_dict.get(fname, "")
            if not material and section:
                # 从截面获取材料 (带缓存)
                if section not in section_material_cache:
                    section_material_cache[section] = _get_section_material(model, section)
                material = section_material_cache[section]
            
            # 获取截面标准化名称 (带缓存)
            standard_section = section
            if section:
                if section not in section_name_cache:
                    section_name_cache[section] = get_standard_section_name(model, section)
                standard_section = section_name_cache[section]
            
            result.append(FrameData(
                name=fname,
                point_i=point_i,
                point_j=point_j,
                length=length,
                material=material,
                section=standard_section
            ))
        
        return result
    finally:
        Units.set_present_units(model, original_units)


def get_point_data(model, point_names: List[str] = None) -> List[PointData]:
    """
    获取节点数据
    
    Args:
        model: SapModel 对象
        point_names: 节点名称列表，None 表示全部
    """
    from global_parameters.units import Units, UnitSystem
    
    # 切换到 N-m-C 单位
    original_units = Units.get_present_units(model)
    Units.set_present_units(model, UnitSystem.N_M_C)
    
    try:
        # 获取节点列表
        if point_names is None:
            ret = model.PointObj.GetNameList(0, [])
            if isinstance(ret, (list, tuple)) and len(ret) >= 2:
                point_names = list(ret[1]) if ret[1] else []
            else:
                point_names = []
        
        result = []
        for pname in point_names:
            ret = model.PointObj.GetCoordCartesian(str(pname), 0.0, 0.0, 0.0)
            if isinstance(ret, (list, tuple)) and len(ret) >= 3:
                result.append(PointData(
                    name=pname,
                    x=ret[0],
                    y=ret[1],
                    z=ret[2]
                ))
        
        return result
    finally:
        Units.set_present_units(model, original_units)


def _get_frame_length(model, frame_name: str) -> float:
    """计算杆件长度 (mm)"""
    ret = model.FrameObj.GetPoints(str(frame_name), "", "")
    if not isinstance(ret, (list, tuple)) or len(ret) < 2:
        return 0.0
    
    point_i, point_j = ret[0], ret[1]
    
    ret1 = model.PointObj.GetCoordCartesian(point_i, 0.0, 0.0, 0.0)
    ret2 = model.PointObj.GetCoordCartesian(point_j, 0.0, 0.0, 0.0)
    
    if isinstance(ret1, (list, tuple)) and isinstance(ret2, (list, tuple)):
        dx = ret2[0] - ret1[0]
        dy = ret2[1] - ret1[1]
        dz = ret2[2] - ret1[2]
        return math.sqrt(dx*dx + dy*dy + dz*dz)
    return 0.0


def _get_section_material(model, section_name: str) -> str:
    """获取截面材料"""
    if not section_name:
        return ""
    
    try:
        # 先获取截面类型
        ret = model.PropFrame.GetTypeOAPI(section_name)
        if not isinstance(ret, (list, tuple)) or len(ret) < 1:
            return ""
        sec_type = ret[0]
        
        # 根据类型调用对应的获取方法
        if sec_type == 8:  # RECTANGULAR
            ret = model.PropFrame.GetRectangle(section_name)
        elif sec_type == 9:  # CIRCLE
            ret = model.PropFrame.GetCircle(section_name)
        elif sec_type == 7:  # PIPE
            ret = model.PropFrame.GetPipe(section_name)
        elif sec_type == 6:  # BOX
            ret = model.PropFrame.GetTube_1(section_name)
        elif sec_type == 1:  # I_SECTION
            ret = model.PropFrame.GetISection_1(section_name)
        elif sec_type == 4:  # ANGLE
            ret = model.PropFrame.GetAngle_1(section_name)
        elif sec_type == 2:  # CHANNEL
            ret = model.PropFrame.GetChannel_2(section_name)
        elif sec_type == 3:  # T_SECTION
            ret = model.PropFrame.GetTee_1(section_name)
        elif sec_type == 5:  # DOUBLE_ANGLE
            ret = model.PropFrame.GetDblAngle_2(section_name)
        elif sec_type == 11:  # DOUBLE_CHANNEL
            ret = model.PropFrame.GetDblChannel_1(section_name)
        else:
            return ""
        
        if isinstance(ret, (list, tuple)) and len(ret) >= 2:
            return ret[1] or ""
    except Exception:
        pass
    return ""


def get_standard_section_name(model, section_name: str) -> str:
    """
    获取截面的标准化命名
    
    使用 FrameSection.standard_name 属性获取标准化名称。
    
    Args:
        model: SapModel 对象
        section_name: 截面名称
    
    Returns:
        标准化的截面名称，失败返回原名称
    
    Note:
        调用前需确保单位为 N-mm-C，API 返回的尺寸单位为 mm
    """
    if not section_name:
        return ""
    
    try:
        from section.frame_section import FrameSection
        section = FrameSection.get_by_name(model, section_name)
        return section.standard_name
    except Exception:
        return section_name


# =============================================================================
# DXF 导出
# =============================================================================

# 模板文件路径
TEMPLATE_DIR = os.path.join(os.path.dirname(__file__), "templates")
DXF_TEMPLATE = os.path.join(TEMPLATE_DIR, "template.dxf")

# 模板中的图层名
LAYER_TABLE = "jointtable"   # 表格外框图层
LAYER_TEXT = "jointtext"     # 文字图层

# 模板中的文字样式
TEXT_STYLE = "毫米(大)"


def _read_template(template_path: str) -> Tuple[bytes, bytes]:
    """
    读取 DXF 模板文件，分离 ENTITIES 段前后的内容 (二进制模式)
    
    模板 ENTITIES 段格式:
    SECTION
      2
    ENTITIES
      0        <-- 这里插入实体
    ENDSEC
    
    Returns:
        (before_entities, after_entities) - ENTITIES 段前后的内容 (bytes)
    """
    with open(template_path, "rb") as f:
        content = f.read()
    
    # 查找 "ENTITIES" 位置
    entities_pos = content.find(b"ENTITIES")
    if entities_pos == -1:
        raise ValueError("模板文件中未找到 ENTITIES 段")
    
    # 找 ENTITIES 后第一个换行
    newline_after_entities = content.find(b"\n", entities_pos)
    if newline_after_entities == -1:
        raise ValueError("模板文件格式错误")
    
    # before: 包含到 ENTITIES 后的换行
    before = content[:newline_after_entities + 1]
    
    # after: 从 ENTITIES 后的 "  0\r\nENDSEC" 或 "  0\nENDSEC" 开始
    after = content[newline_after_entities + 1:]
    
    return before, after


# 全局句柄计数器
_handle_counter = 0x2000


def _next_handle() -> str:
    """获取下一个句柄 (十六进制)"""
    global _handle_counter
    _handle_counter += 1
    return format(_handle_counter, 'X')


def _reset_handle():
    """重置句柄计数器"""
    global _handle_counter
    _handle_counter = 0x2000


def _generate_dxf_text(x: float, y: float, text: str, height: float = 350, 
                       layer: str = LAYER_TEXT, style: str = TEXT_STYLE,
                       use_ac1024: bool = True) -> str:
    """生成 DXF TEXT 实体"""
    if use_ac1024:
        # AC1024 格式 (AutoCAD 2010+)
        handle = _next_handle()
        return f"""  0
TEXT
  5
{handle}
330
6F
100
AcDbEntity
  8
{layer}
100
AcDbText
 10
{x:.6f}
 20
{y:.6f}
 30
0.0
 40
{height:.6f}
  1
{text}
  7
{style}
100
AcDbText
"""
    else:
        # R12 格式
        return f"""  0
TEXT
  8
{layer}
 10
{x:.6f}
 20
{y:.6f}
 30
0.0
 40
{height:.6f}
  1
{text}
  7
{style}
"""


def _generate_dxf_line(x1: float, y1: float, x2: float, y2: float, 
                       layer: str = LAYER_TABLE,
                       use_ac1024: bool = True) -> str:
    """生成 DXF LINE 实体"""
    if use_ac1024:
        # AC1024 格式 (AutoCAD 2010+)
        handle = _next_handle()
        return f"""  0
LINE
  5
{handle}
330
6F
100
AcDbEntity
  8
{layer}
100
AcDbLine
 10
{x1:.6f}
 20
{y1:.6f}
 30
0.0
 11
{x2:.6f}
 21
{y2:.6f}
 31
0.0
"""
    else:
        # R12 格式
        return f"""  0
LINE
  8
{layer}
 10
{x1:.6f}
 20
{y1:.6f}
 30
0.0
 11
{x2:.6f}
 21
{y2:.6f}
 31
0.0
"""


def _write_dxf_header(f):
    """写入 DXF 文件头 (无模板时使用)"""
    f.write("0\nSECTION\n2\nENTITIES\n")


def _write_dxf_footer(f):
    """写入 DXF 文件尾 (无模板时使用)"""
    f.write("0\nENDSEC\n0\nEOF\n")


def _write_dxf_text(f, x: float, y: float, text: str, height: float = 350, 
                    color: int = 3, layer: str = "text"):
    """写入 DXF 文本 (无模板时使用)"""
    f.write(f"0\nTEXT\n8\n{layer}\n62\n{color}\n10\n{x}\n20\n{y}\n30\n0\n40\n{height}\n1\n{text}\n")


def _write_dxf_line(f, x1: float, y1: float, x2: float, y2: float, 
                    color: int = 5, layer: str = "line"):
    """写入 DXF 线段 (无模板时使用)"""
    f.write(f"0\nLINE\n8\n{layer}\n62\n{color}\n10\n{x1}\n20\n{y1}\n30\n0\n11\n{x2}\n21\n{y2}\n31\n0\n")


def export_frame_table(
    model,
    output_file: str,
    selected_only: bool = False,
    rows_per_column: int = 90,
    row_height: float = 800,
    text_height: float = 350,
    column_gap: float = 2500,
    group_gap: float = 40000,
    tables_per_group: int = 5,
    use_template: bool = True
) -> int:
    """
    导出杆件表格到 DXF
    
    Args:
        model: SapModel 对象
        output_file: 输出文件路径 (.dxf)
        selected_only: 是否只导出选中的杆件
        rows_per_column: 每列行数 (默认90)
        row_height: 行高
        text_height: 文字高度
        column_gap: 组内列间距 (默认2500)
        group_gap: 组间间距 (默认40000)
        tables_per_group: 每组表格数 (默认5)
        use_template: 是否使用模板 (默认True)
        
    Returns:
        导出的杆件数量
    """
    # 获取杆件数据
    if selected_only:
        frame_names, _ = get_selected_objects(model)
        print(f"选中的杆件: {frame_names}")
    else:
        frame_names = None
    
    frames = get_frame_data(model, frame_names)
    if not frames:
        print("没有杆件数据")
        return 0
    
    # 按单元号排序 (尝试数字排序，失败则字符串排序)
    def sort_key(f):
        try:
            return (0, int(f.name))
        except ValueError:
            return (1, f.name)
    frames.sort(key=sort_key)
    
    # 重置句柄计数器
    _reset_handle()
    
    # 列宽定义 (单元号、节点1、节点2、长度、材料、截面)
    col_widths = [2400, 2400, 2400, 2400, 2400, 4800]
    headers = ["单元号", "节点1", "节点2", "长度(mm)", "材料", "截面"]
    table_width = sum(col_widths)
    
    # 计算需要多少个表格列
    num_tables = (len(frames) + rows_per_column - 1) // rows_per_column
    
    # 生成实体内容
    entities_content = []
    
    for table_idx in range(num_tables):
        # 计算当前表格的起始位置
        # 每 tables_per_group 列为一组，组内间距 column_gap，组间间距 group_gap
        group_idx = table_idx // tables_per_group  # 第几组
        idx_in_group = table_idx % tables_per_group  # 组内第几个
        
        # x_offset = 组偏移 + 组内偏移
        group_offset = group_idx * (tables_per_group * (table_width + column_gap) - column_gap + group_gap)
        in_group_offset = idx_in_group * (table_width + column_gap)
        x_offset = group_offset + in_group_offset
        
        # 当前表格的数据范围
        start_idx = table_idx * rows_per_column
        end_idx = min(start_idx + rows_per_column, len(frames))
        table_frames = frames[start_idx:end_idx]
        
        # 表格高度
        table_height = (len(table_frames) + 1) * row_height
        
        # 绘制水平线
        y = 0
        for i in range(len(table_frames) + 2):
            entities_content.append(_generate_dxf_line(x_offset, y, x_offset + table_width, y, use_ac1024=use_template))
            y -= row_height
        
        # 绘制垂直线
        x = x_offset
        for w in col_widths:
            entities_content.append(_generate_dxf_line(x, 0, x, -table_height, use_ac1024=use_template))
            x += w
        entities_content.append(_generate_dxf_line(x_offset + table_width, 0, x_offset + table_width, -table_height, use_ac1024=use_template))
        
        # 写入表头
        x = x_offset + 50
        for i, header in enumerate(headers):
            entities_content.append(_generate_dxf_text(x, -row_height + 200, header, text_height, use_ac1024=use_template))
            x += col_widths[i]
        
        # 写入数据
        y = -row_height
        for frame in table_frames:
            y -= row_height
            row_data = [
                frame.name,
                frame.point_i,
                frame.point_j,
                f"{frame.length:.0f}",
                frame.material,
                frame.section
            ]
            x = x_offset + 50
            for i, data in enumerate(row_data):
                entities_content.append(_generate_dxf_text(x, y + 200, str(data), text_height, use_ac1024=use_template))
                x += col_widths[i]
    
    # 添加截面说明 (在第一个表格下方，分2列)
    legend_y = -(rows_per_column + 3) * row_height
    legend_col1 = [
        "截面说明:",
        "H - I形截面 (高x宽x腹板厚x翼缘厚)",
        "B - 箱形截面 (高x宽x腹板厚x翼缘厚)",
        "P - 圆管 (直径x壁厚)",
        "R - 矩形截面 (高x宽)",
        "D - 圆形截面 (直径)",
        
       
    ]
    legend_col2 = [
        "",
        "T - T形截面 (高x宽x腹板厚x翼缘厚)",
        "C - 槽钢 (高x宽x腹板厚x翼缘厚)",
        "L - 角钢 (高x宽x厚)",
        "2C - 双槽钢 (高x宽x厚x间距)",
        "2L - 双角钢 (高x宽x厚x间距)",
    ]
    col2_x = 9000  # 第二列x偏移
    for i, line in enumerate(legend_col1):
        entities_content.append(_generate_dxf_text(0, legend_y - i * row_height, line, text_height, use_ac1024=use_template))
    for i, line in enumerate(legend_col2):
        if line:
            entities_content.append(_generate_dxf_text(col2_x, legend_y - i * row_height, line, text_height, use_ac1024=use_template))
    
    # 写入文件
    if use_template and os.path.exists(DXF_TEMPLATE):
        # 使用模板 (二进制模式，UTF-8 编码)
        before, after = _read_template(DXF_TEMPLATE)
        entities_bytes = "".join(entities_content).encode("utf-8")
        with open(output_file, "wb") as f:
            f.write(before)
            f.write(entities_bytes)
            f.write(after)
    else:
        # 不使用模板
        with open(output_file, "w", encoding="gbk") as f:
            _write_dxf_header(f)
            for entity in entities_content:
                f.write(entity)
            _write_dxf_footer(f)
    
    print(f"导出完成: {len(frames)} 根杆件 -> {output_file}")
    return len(frames)


def export_point_table(
    model,
    output_file: str,
    selected_only: bool = False,
    rows_per_column: int = 90,
    row_height: float = 800,
    text_height: float = 350,
    column_gap: float = 2500,
    group_gap: float = 40000,
    tables_per_group: int = 4,
    use_template: bool = True
) -> int:
    """
    导出节点表格到 DXF
    
    Args:
        model: SapModel 对象
        output_file: 输出文件路径 (.dxf)
        selected_only: 是否只导出选中的节点
        rows_per_column: 每列行数 (默认90)
        row_height: 行高
        text_height: 文字高度
        column_gap: 组内列间距 (默认2500)
        group_gap: 组间间距 (默认40000)
        tables_per_group: 每组表格数 (默认4)
        use_template: 是否使用模板 (默认True)
        
    Returns:
        导出的节点数量
    """
    # 获取节点数据
    if selected_only:
        _, point_names = get_selected_objects(model)
    else:
        point_names = None
    
    points = get_point_data(model, point_names)
    if not points:
        print("没有节点数据")
        return 0
    
    # 按节点号排序 (尝试数字排序，失败则字符串排序)
    def sort_key(p):
        try:
            return (0, int(p.name))
        except ValueError:
            return (1, p.name)
    points.sort(key=sort_key)
    
    # 重置句柄计数器
    _reset_handle()
    
    # 列宽定义 (节点号、X、Y、Z)
    col_widths = [1500, 2500, 2500, 2500]
    headers = ["节点号", "X(m)", "Y(m)", "Z(m)"]
    table_width = sum(col_widths)
    
    # 计算需要多少个表格列
    num_tables = (len(points) + rows_per_column - 1) // rows_per_column
    
    # 生成实体内容
    entities_content = []
    
    for table_idx in range(num_tables):
        # 计算当前表格的起始位置
        # 每 tables_per_group 列为一组，组内间距 column_gap，组间间距 group_gap
        group_idx = table_idx // tables_per_group
        idx_in_group = table_idx % tables_per_group
        
        group_offset = group_idx * (tables_per_group * (table_width + column_gap) - column_gap + group_gap)
        in_group_offset = idx_in_group * (table_width + column_gap)
        x_offset = group_offset + in_group_offset
        
        # 当前表格的数据范围
        start_idx = table_idx * rows_per_column
        end_idx = min(start_idx + rows_per_column, len(points))
        table_points = points[start_idx:end_idx]
        
        # 表格高度
        table_height = (len(table_points) + 1) * row_height
        
        # 绘制水平线
        y = 0
        for i in range(len(table_points) + 2):
            entities_content.append(_generate_dxf_line(x_offset, y, x_offset + table_width, y, use_ac1024=use_template))
            y -= row_height
        
        # 绘制垂直线
        x = x_offset
        for w in col_widths:
            entities_content.append(_generate_dxf_line(x, 0, x, -table_height, use_ac1024=use_template))
            x += w
        entities_content.append(_generate_dxf_line(x_offset + table_width, 0, x_offset + table_width, -table_height, use_ac1024=use_template))
        
        # 写入表头
        x = x_offset + 50
        for i, header in enumerate(headers):
            entities_content.append(_generate_dxf_text(x, -row_height + 200, header, text_height, use_ac1024=use_template))
            x += col_widths[i]
        
        # 写入数据
        y = -row_height
        for point in table_points:
            y -= row_height
            row_data = [
                point.name,
                f"{point.x:.3f}",
                f"{point.y:.3f}",
                f"{point.z:.3f}"
            ]
            x = x_offset + 50
            for i, data in enumerate(row_data):
                entities_content.append(_generate_dxf_text(x, y + 200, str(data), text_height, use_ac1024=use_template))
                x += col_widths[i]
    
    # 写入文件
    if use_template and os.path.exists(DXF_TEMPLATE):
        # 使用模板 (二进制模式，UTF-8 编码)
        before, after = _read_template(DXF_TEMPLATE)
        entities_bytes = "".join(entities_content).encode("utf-8")
        with open(output_file, "wb") as f:
            f.write(before)
            f.write(entities_bytes)
            f.write(after)
    else:
        # 不使用模板
        with open(output_file, "w", encoding="gbk") as f:
            _write_dxf_header(f)
            for entity in entities_content:
                f.write(entity)
            _write_dxf_footer(f)
    
    print(f"导出完成: {len(points)} 个节点 -> {output_file}")
    return len(points)


def export_to_csv(
    model,
    output_file: str,
    export_type: str = "frame",
    selected_only: bool = False
) -> int:
    """
    导出到 CSV 文件
    
    Args:
        model: SapModel 对象
        output_file: 输出文件路径 (.csv)
        export_type: "frame" 或 "point"
        selected_only: 是否只导出选中的对象
        
    Returns:
        导出的数量
    """
    if export_type == "frame":
        if selected_only:
            frame_names, _ = get_selected_objects(model)
        else:
            frame_names = None
        
        frames = get_frame_data(model, frame_names)
        if not frames:
            print("没有杆件数据")
            return 0
        
        with open(output_file, "w", encoding="utf-8-sig") as f:
            f.write("单元号,节点1,节点2,长度(mm),材料,截面\n")
            for frame in frames:
                f.write(f"{frame.name},{frame.point_i},{frame.point_j},"
                       f"{frame.length:.0f},{frame.material},{frame.section}\n")
        
        print(f"导出完成: {len(frames)} 根杆件 -> {output_file}")
        return len(frames)
    
    elif export_type == "point":
        if selected_only:
            _, point_names = get_selected_objects(model)
        else:
            point_names = None
        
        points = get_point_data(model, point_names)
        if not points:
            print("没有节点数据")
            return 0
        
        with open(output_file, "w", encoding="utf-8-sig") as f:
            f.write("节点号,X(m),Y(m),Z(m)\n")
            for point in points:
                f.write(f"{point.name},{point.x:.3f},{point.y:.3f},{point.z:.3f}\n")
        
        print(f"导出完成: {len(points)} 个节点 -> {output_file}")
        return len(points)
    
    return 0


def _generate_dxf_line_3d(x1: float, y1: float, z1: float, 
                          x2: float, y2: float, z2: float,
                          layer: str = "frame",
                          use_ac1024: bool = True) -> str:
    """生成 3D DXF LINE 实体"""
    if use_ac1024:
        handle = _next_handle()
        return f"""  0
LINE
  5
{handle}
330
6F
100
AcDbEntity
  8
{layer}
100
AcDbLine
 10
{x1:.6f}
 20
{y1:.6f}
 30
{z1:.6f}
 11
{x2:.6f}
 21
{y2:.6f}
 31
{z2:.6f}
"""
    else:
        return f"""  0
LINE
  8
{layer}
 10
{x1:.6f}
 20
{y1:.6f}
 30
{z1:.6f}
 11
{x2:.6f}
 21
{y2:.6f}
 31
{z2:.6f}
"""


def _generate_dxf_text_3d(x: float, y: float, z: float, text: str, 
                          height: float = 350, layer: str = "text",
                          color: int = None, style: str = TEXT_STYLE,
                          use_ac1024: bool = True) -> str:
    """生成 3D DXF TEXT 实体
    
    Args:
        color: ACI颜色索引 (1=红, 2=黄, 3=绿, 4=青, 5=蓝, 6=洋红, 7=白, 30=橙)
        style: 文字样式名称
    """
    color_line = f" 62\n{color}\n" if color else ""
    if use_ac1024:
        handle = _next_handle()
        return f"""  0
TEXT
  5
{handle}
330
6F
100
AcDbEntity
  8
{layer}
{color_line}100
AcDbText
 10
{x:.6f}
 20
{y:.6f}
 30
{z:.6f}
 40
{height:.6f}
  1
{text}
  7
{style}
100
AcDbText
"""
    else:
        return f"""  0
TEXT
  8
{layer}
{color_line} 10
{x:.6f}
 20
{y:.6f}
 30
{z:.6f}
 40
{height:.6f}
  1
{text}
  7
{style}
"""


def _project_point(x: float, y: float, z: float, projection) -> Tuple[float, float, float]:
    """
    将3D坐标投影到2D平面，与 ModelViewer 中的视角一致
    
    坐标系说明:
    - SAP2000: X右, Y前(屏幕内), Z上
    - Three.js/ModelViewer: X右, Y上, Z前(屏幕外)
    - ModelViewer 转换: three_x=sap_x, three_y=sap_z, three_z=-sap_y
    
    Args:
        x, y, z: 3D坐标 (SAP2000坐标系)
        projection: 投影类型
    
    Returns:
        (x2d, y2d, z2d) - 投影后的坐标
    """
    # 处理自定义角度（带视图矩阵）
    if isinstance(projection, dict):
        # 先将 SAP 坐标转换到 Three.js 坐标系
        # ModelViewer: three_x=sap_x, three_y=sap_z, three_z=-sap_y
        tx = x
        ty = z
        tz = -y
        
        # 优先使用视图矩阵
        vm = projection.get("viewMatrix")
        if vm and len(vm) >= 6:
            # vm = [right.x, right.y, right.z, orthoUp.x, orthoUp.y, orthoUp.z]
            # right 和 orthoUp 是 Three.js 坐标系中的向量
            # 投影: screen_x = right · point, screen_y = orthoUp · point
            px = vm[0] * tx + vm[1] * ty + vm[2] * tz
            py = vm[3] * tx + vm[4] * ty + vm[5] * tz
            return px, py, 0.0
        
        # 如果没有视图矩阵，使用角度计算
        angles = projection.get("angles")
        if angles:
            rx_deg = angles.get("x", 0)
            ry_deg = angles.get("y", 0)
            rx = math.radians(rx_deg)
            ry = math.radians(ry_deg)
            
            # 绕Y轴旋转 -ry
            c, s = math.cos(-ry), math.sin(-ry)
            tx1 = tx * c + tz * s
            ty1 = ty
            tz1 = -tx * s + tz * c
            
            # 绕X轴旋转 -rx
            c, s = math.cos(-rx), math.sin(-rx)
            tx2 = tx1
            ty2 = ty1 * c - tz1 * s
            
            return tx2, ty2, 0.0
    
    if projection == "3d":
        return x, y, z
    elif projection == "iso":
        # 等轴测投影: 先转到Three.js坐标，再应用等轴测
        tx, ty, tz = x, z, -y
        ry = math.radians(45)
        rx = math.radians(35.264)
        
        c, s = math.cos(-ry), math.sin(-ry)
        tx1 = tx * c + tz * s
        ty1 = ty
        tz1 = -tx * s + tz * c
        
        c, s = math.cos(-rx), math.sin(-rx)
        tx2 = tx1
        ty2 = ty1 * c - tz1 * s
        
        return tx2, ty2, 0.0
    elif projection == "top":
        # 俯视图: 从上往下看 (rx=90, ry=0)
        return x, -y, 0.0
    elif projection == "front":
        # 正视图: 从Y正方向看 (rx=0, ry=0)
        return x, z, 0.0
    elif projection == "right":
        # 右视图: 从X正方向看 (rx=0, ry=90)
        return -y, z, 0.0
    else:
        return x, y, z


def export_3d_model(
    model,
    output_file: str,
    selected_only: bool = False,
    group_names: List[str] = None,
    group_gap: float = 400000,
    show_point_labels: bool = False,
    show_frame_labels: bool = False,
    label_height: float = 350,
    projection = "3d",
    use_template: bool = True
) -> int:
    """
    导出结构三维线框模型到 DXF
    
    Args:
        model: SapModel 对象
        output_file: 输出文件路径 (.dxf)
        selected_only: 是否只导出选中的杆件
        group_names: 组名列表，按顺序在X方向排列，None表示导出全部
        group_gap: 组间间距 (默认400000mm)
        show_point_labels: 是否显示节点编号
        show_frame_labels: 是否显示单元编号
        label_height: 标签文字高度 (默认350mm)
        projection: 投影类型，可以是:
            - 字符串: 所有组使用相同投影 ("3d", "iso", "top", "front", "right")
            - 列表: 每个组使用不同投影，如 ["iso", "top", "front"]
        use_template: 是否使用模板 (默认True)
        
    Returns:
        导出的杆件数量
    """
    from global_parameters.units import Units, UnitSystem
    
    # 切换到 N-mm-C 单位
    original_units = Units.get_present_units(model)
    Units.set_present_units(model, UnitSystem.N_MM_C)
    
    try:
        # 获取所有节点坐标
        point_coords = {}  # point_name -> (x, y, z)
        ret = model.PointObj.GetNameList(0, [])
        if isinstance(ret, (list, tuple)) and len(ret) >= 2:
            point_names = list(ret[1]) if ret[1] else []
            for pname in point_names:
                ret = model.PointObj.GetCoordCartesian(str(pname), 0.0, 0.0, 0.0)
                if isinstance(ret, (list, tuple)) and len(ret) >= 3:
                    point_coords[pname] = (ret[0], ret[1], ret[2])
        
        # 重置句柄计数器
        _reset_handle()
        
        # 生成实体内容
        entities_content = []
        total_count = 0
        labeled_points = set()  # 已标注的节点
        
        if group_names:
            # 按组导出
            x_offset = 0.0
            for group_idx, group_name in enumerate(group_names):
                # 获取该组的投影类型
                if isinstance(projection, list):
                    group_projection = projection[group_idx] if group_idx < len(projection) else "3d"
                else:
                    group_projection = projection
                
                # 获取组内杆件
                ret = model.GroupDef.GetAssignments(group_name, 0, [], [])
                if not isinstance(ret, (list, tuple)) or len(ret) < 3:
                    print(f"警告: 无法获取组 '{group_name}' 的成员")
                    continue
                
                obj_types = ret[1] if ret[1] else []
                obj_names = ret[2] if ret[2] else []
                
                # 筛选杆件 (type=2)
                frame_names = [obj_names[i] for i, t in enumerate(obj_types) if t == 2]
                
                if not frame_names:
                    print(f"警告: 组 '{group_name}' 中没有杆件")
                    continue
                
                # 计算该组投影后的边界框
                min_px, max_px = float('inf'), float('-inf')
                min_py, max_py = float('inf'), float('-inf')
                for fname in frame_names:
                    ret = model.FrameObj.GetPoints(str(fname), "", "")
                    if isinstance(ret, (list, tuple)) and len(ret) >= 2:
                        for pname in [ret[0], ret[1]]:
                            if pname in point_coords:
                                x, y, z = point_coords[pname]
                                px, py, _ = _project_point(x, y, z, group_projection)
                            else:
                                continue
                            min_px = min(min_px, px)
                            max_px = max(max_px, px)
                            min_py = min(min_py, py)
                            max_py = max(max_py, py)
                
                group_width = max_px - min_px if max_px > min_px else 0
                
                # 绘制该组的杆件
                count = 0
                group_labeled_points = set()  # 该组已标注的节点
                for fname in frame_names:
                    ret = model.FrameObj.GetPoints(str(fname), "", "")
                    if not isinstance(ret, (list, tuple)) or len(ret) < 2:
                        continue
                    
                    point_i, point_j = ret[0], ret[1]
                    
                    # 获取投影坐标
                    if point_i not in point_coords or point_j not in point_coords:
                        continue
                    x1, y1, z1 = point_coords[point_i]
                    x2, y2, z2 = point_coords[point_j]
                    px1, py1, pz1 = _project_point(x1, y1, z1, group_projection)
                    px2, py2, pz2 = _project_point(x2, y2, z2, group_projection)
                    
                    # 应用偏移 (相对于投影后的边界框)
                    px1 = px1 - min_px + x_offset
                    px2 = px2 - min_px + x_offset
                    py1 = py1 - min_py
                    py2 = py2 - min_py
                    
                    entities_content.append(_generate_dxf_line_3d(
                        px1, py1, pz1, px2, py2, pz2,
                        layer="frame", use_ac1024=use_template
                    ))
                    
                    # 添加单元编号 (在线段中点)
                    if show_frame_labels:
                        mid_x = (px1 + px2) / 2
                        mid_y = (py1 + py2) / 2
                        mid_z = (pz1 + pz2) / 2
                        entities_content.append(_generate_dxf_text_3d(
                            mid_x, mid_y, mid_z, str(fname), label_height,
                            layer="jointtext", use_ac1024=use_template
                        ))
                    
                    # 添加节点编号 (在节点位置，避免重复)
                    if show_point_labels:
                        if point_i not in group_labeled_points:
                            entities_content.append(_generate_dxf_text_3d(
                                px1, py1, pz1, str(point_i), label_height,
                                layer="jointtable", use_ac1024=use_template
                            ))
                            group_labeled_points.add(point_i)
                        if point_j not in group_labeled_points:
                            entities_content.append(_generate_dxf_text_3d(
                                px2, py2, pz2, str(point_j), label_height,
                                layer="jointtable", use_ac1024=use_template
                            ))
                            group_labeled_points.add(point_j)
                    
                    count += 1
                
                print(f"组 '{group_name}': {count} 根杆件")
                total_count += count
                
                # 更新下一组的X偏移
                x_offset += group_width + group_gap
        else:
            # 导出全部或选中
            if selected_only:
                frame_names, _ = get_selected_objects(model)
            else:
                ret = model.FrameObj.GetNameList(0, [])
                if isinstance(ret, (list, tuple)) and len(ret) >= 2:
                    frame_names = list(ret[1]) if ret[1] else []
                else:
                    frame_names = []
            
            if not frame_names:
                print("没有杆件数据")
                return 0
            
            for fname in frame_names:
                ret = model.FrameObj.GetPoints(str(fname), "", "")
                if not isinstance(ret, (list, tuple)) or len(ret) < 2:
                    continue
                
                point_i, point_j = ret[0], ret[1]
                if point_i not in point_coords or point_j not in point_coords:
                    continue
                
                x1, y1, z1 = point_coords[point_i]
                x2, y2, z2 = point_coords[point_j]
                
                # 应用投影
                px1, py1, pz1 = _project_point(x1, y1, z1, projection)
                px2, py2, pz2 = _project_point(x2, y2, z2, projection)
                
                entities_content.append(_generate_dxf_line_3d(
                    px1, py1, pz1, px2, py2, pz2,
                    layer="frame", use_ac1024=use_template
                ))
                
                # 添加单元编号 (在线段中点)
                if show_frame_labels:
                    mid_x = (px1 + px2) / 2
                    mid_y = (py1 + py2) / 2
                    mid_z = (pz1 + pz2) / 2
                    entities_content.append(_generate_dxf_text_3d(
                        mid_x, mid_y, mid_z, str(fname), label_height,
                        layer="jointtext", use_ac1024=use_template
                    ))
                
                # 添加节点编号 (在节点位置，避免重复)
                if show_point_labels:
                    if point_i not in labeled_points:
                        entities_content.append(_generate_dxf_text_3d(
                            px1, py1, pz1, str(point_i), label_height,
                            layer="jointtable", use_ac1024=use_template
                        ))
                        labeled_points.add(point_i)
                    if point_j not in labeled_points:
                        entities_content.append(_generate_dxf_text_3d(
                            px2, py2, pz2, str(point_j), label_height,
                            layer="jointtable", use_ac1024=use_template
                        ))
                        labeled_points.add(point_j)
                
                total_count += 1
        
        if total_count == 0:
            print("没有有效的杆件数据")
            return 0
        
        # 写入文件
        if use_template and os.path.exists(DXF_TEMPLATE):
            before, after = _read_template(DXF_TEMPLATE)
            entities_bytes = "".join(entities_content).encode("utf-8")
            with open(output_file, "wb") as f:
                f.write(before)
                f.write(entities_bytes)
                f.write(after)
        else:
            with open(output_file, "w", encoding="gbk") as f:
                _write_dxf_header(f)
                for entity in entities_content:
                    f.write(entity)
                _write_dxf_footer(f)
        
        print(f"导出完成: {total_count} 根杆件 -> {output_file}")
        return total_count
    finally:
        Units.set_present_units(model, original_units)


# =============================================================================
# 使用示例
# =============================================================================

if __name__ == "__main__":
    import sys
    import os
    sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
    
    try:
        from application import Application
        
        app = Application()
        model = app.model
        
        # 导出三维线框模型 (按组，每组不同视角)
        export_3d_model(
            model, "model_3d.dxf", 
            group_names=["01-外环桁架", "02-内圈桁架", "03-外立面构件"],
            projection=["iso", "iso", "iso"],
            show_point_labels=True,
            show_frame_labels=False
        )
        
        # 导出三维线框模型 (全部)
        # export_3d_model(model, "model_3d.dxf", selected_only=False)
        
        # 导出杆件表格到 DXF
        # export_frame_table(model, "frames_out.dxf", selected_only=False, use_template=True)
        
        # 导出节点表格到 DXF
        export_point_table(model, "points_out.dxf", selected_only=True, use_template=True)
        
    except Exception as e:
        print(f"错误: {e}")
        import traceback
        traceback.print_exc()



# -*- coding: utf-8 -*-
"""
frame - 杆件属性相关类型和函数

为 AI Agent 设计的模块化 API，按功能分类：

1. 端部释放 (release)
   - set_frame_release: 设置释放类型 (两端固定/I端铰接/J端铰接/两端铰接)
   - set_frame_release_custom: 设置自定义释放 (6个自由度)
   - get_frame_release: 获取释放状态
   - get_frame_release_type: 获取释放类型
   - is_frame_hinged: 检查是否铰接

2. 局部坐标轴 (local_axes)
   - set_frame_local_axes: 设置局部轴角度
   - get_frame_local_axes: 获取局部轴角度
   - get_frame_transformation_matrix: 获取变换矩阵

3. 截面修改器 (modifier)
   - set_frame_modifiers: 设置修改器
   - get_frame_modifiers: 获取修改器
   - delete_frame_modifiers: 删除修改器

4. 质量 (mass)
   - set_frame_mass: 设置单位长度质量
   - get_frame_mass: 获取单位长度质量
   - delete_frame_mass: 删除附加质量

5. 选择 (selection)
   - set_frame_selected: 设置选择状态
   - select_frame/deselect_frame: 选中/取消选中

6. 组分配 (group)
   - add_frame_to_group: 添加到组
   - remove_frame_from_group: 从组移除
   - get_frame_groups: 获取所属组

7. 属性分配 (property)
   - set_frame_section: 设置截面属性
   - get_frame_section: 获取截面属性

注意: 杆件荷载相关函数在 loads 模块
"""

# ==================== 枚举类型 ====================
from .enums import (
    FrameType,
    FrameSectionType,
    FrameReleaseType,
    ItemType,
    SECTION_TYPE_NAMES,
    RELEASE_PRESETS,
)

# ==================== 数据类 ====================
from .data_classes import (
    FrameReleaseData,
    FrameModifierData,
    FrameLocalAxesData,
    FrameMassData,
)

# ==================== 端部释放函数 ====================
from .release import (
    set_frame_release,
    set_frame_release_custom,
    get_frame_release,
    get_frame_release_type,
    is_frame_hinged,
)

# ==================== 局部坐标轴函数 ====================
from .local_axes import (
    set_frame_local_axes,
    get_frame_local_axes,
    get_frame_transformation_matrix,
)

# ==================== 修改器函数 ====================
from .modifier import (
    set_frame_modifiers,
    set_frame_modifiers_tuple,
    get_frame_modifiers,
    get_frame_modifiers_tuple,
    delete_frame_modifiers,
)

# ==================== 质量函数 ====================
from .mass import (
    set_frame_mass,
    get_frame_mass,
    get_frame_mass_data,
    delete_frame_mass,
    has_frame_mass,
)

# ==================== 选择函数 ====================
from .selection import (
    set_frame_selected,
    get_frame_selected,
    select_frame,
    deselect_frame,
    select_frames,
    deselect_frames,
    is_frame_selected,
)

# ==================== 组分配函数 ====================
from .group import (
    set_frame_group,
    add_frame_to_group,
    remove_frame_from_group,
    get_frame_groups,
    is_frame_in_group,
    add_frames_to_group,
    remove_frames_from_group,
)

# ==================== 属性分配函数 ====================
from .property import (
    set_frame_section,
    get_frame_section,
    get_frame_section_info,
    set_frame_material_overwrite,
    get_frame_material_overwrite,
    set_frame_material_temperature,
    get_frame_material_temperature,
)

# ==================== 铰数据类 ====================
from .hinge import (
    FrameHinge,
    FrameHingeType,
    HINGE_RELEASES,
)


# ==================== API 分类索引 (供 AI Agent 发现功能) ====================

FRAME_API_CATEGORIES = {
    "端部释放": {
        "description": "设置杆件端部的约束释放（铰接）",
        "functions": [
            "set_frame_release",
            "set_frame_release_custom",
            "get_frame_release",
            "get_frame_release_type",
            "is_frame_hinged",
        ]
    },
    "局部坐标轴": {
        "description": "设置杆件的局部坐标轴方向",
        "functions": [
            "set_frame_local_axes",
            "get_frame_local_axes",
            "get_frame_transformation_matrix",
        ]
    },
    "截面修改器": {
        "description": "设置杆件的截面属性修改器（刚度折减等）",
        "functions": [
            "set_frame_modifiers",
            "set_frame_modifiers_tuple",
            "get_frame_modifiers",
            "get_frame_modifiers_tuple",
            "delete_frame_modifiers",
        ]
    },
    "质量": {
        "description": "设置杆件的附加质量",
        "functions": [
            "set_frame_mass",
            "get_frame_mass",
            "get_frame_mass_data",
            "delete_frame_mass",
            "has_frame_mass",
        ]
    },
    "选择": {
        "description": "设置杆件的选择状态",
        "functions": [
            "set_frame_selected",
            "get_frame_selected",
            "select_frame",
            "deselect_frame",
            "select_frames",
            "deselect_frames",
            "is_frame_selected",
        ]
    },
    "组分配": {
        "description": "设置杆件的组分配",
        "functions": [
            "set_frame_group",
            "add_frame_to_group",
            "remove_frame_from_group",
            "get_frame_groups",
            "is_frame_in_group",
            "add_frames_to_group",
            "remove_frames_from_group",
        ]
    },
    "属性分配": {
        "description": "设置杆件的截面属性和材料",
        "functions": [
            "set_frame_section",
            "get_frame_section",
            "get_frame_section_info",
            "set_frame_material_overwrite",
            "get_frame_material_overwrite",
            "set_frame_material_temperature",
            "get_frame_material_temperature",
        ]
    },
}


# ==================== 导出列表 ====================

__all__ = [
    # 枚举类型
    'FrameType',
    'FrameSectionType',
    'FrameReleaseType',
    'ItemType',
    'SECTION_TYPE_NAMES',
    'RELEASE_PRESETS',
    
    # 数据类
    'FrameReleaseData',
    'FrameModifierData',
    'FrameLocalAxesData',
    'FrameMassData',
    'FrameHinge',
    'FrameHingeType',
    'HINGE_RELEASES',
    
    # 端部释放函数
    'set_frame_release',
    'set_frame_release_custom',
    'get_frame_release',
    'get_frame_release_type',
    'is_frame_hinged',
    
    # 局部坐标轴函数
    'set_frame_local_axes',
    'get_frame_local_axes',
    'get_frame_transformation_matrix',
    
    # 修改器函数
    'set_frame_modifiers',
    'set_frame_modifiers_tuple',
    'get_frame_modifiers',
    'get_frame_modifiers_tuple',
    'delete_frame_modifiers',
    
    # 质量函数
    'set_frame_mass',
    'get_frame_mass',
    'get_frame_mass_data',
    'delete_frame_mass',
    'has_frame_mass',
    
    # 选择函数
    'set_frame_selected',
    'get_frame_selected',
    'select_frame',
    'deselect_frame',
    'select_frames',
    'deselect_frames',
    'is_frame_selected',
    
    # 组分配函数
    'set_frame_group',
    'add_frame_to_group',
    'remove_frame_from_group',
    'get_frame_groups',
    'is_frame_in_group',
    'add_frames_to_group',
    'remove_frames_from_group',
    
    # 属性分配函数
    'set_frame_section',
    'get_frame_section',
    'get_frame_section_info',
    'set_frame_material_overwrite',
    'get_frame_material_overwrite',
    'set_frame_material_temperature',
    'get_frame_material_temperature',
    
    # API 分类索引
    'FRAME_API_CATEGORIES',
]



# -*- coding: utf-8 -*-
"""
mass.py - 杆件质量相关函数

用于设置杆件的附加质量

SAP2000 API:
- FrameObj.SetMass(Name, MassOverL, Replace, ItemType)
- FrameObj.GetMass(Name, MassOverL)
- FrameObj.DeleteMass(Name, ItemType)
"""

from typing import Optional
from .enums import ItemType
from .data_classes import FrameMassData


def set_frame_mass(
    model,
    frame_name: str,
    mass_per_length: float,
    replace: bool = True,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    设置杆件单位长度质量
    
    用于添加非结构质量（如管道、设备等）。
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
        mass_per_length: 单位长度质量 [M/L]
        replace: True=替换现有质量, False=叠加
        item_type: 操作范围
    
    Returns:
        0 表示成功
    
    Example:
        # 设置杆件附加质量 100 kg/m
        set_frame_mass(model, "1", 100)
        
        # 叠加质量
        set_frame_mass(model, "1", 50, replace=False)
    """
    return model.FrameObj.SetMass(str(frame_name), mass_per_length, replace, int(item_type))


def get_frame_mass(
    model,
    frame_name: str
) -> Optional[float]:
    """
    获取杆件单位长度质量
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
    
    Returns:
        单位长度质量 [M/L]，失败返回 None
    
    Example:
        mass = get_frame_mass(model, "1")
        if mass:
            print(f"单位长度质量: {mass}")
    """
    try:
        result = model.FrameObj.GetMass(str(frame_name), 0.0)
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            return result[0]
    except Exception:
        pass
    return None


def get_frame_mass_data(
    model,
    frame_name: str
) -> Optional[FrameMassData]:
    """
    获取杆件质量数据对象
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
    
    Returns:
        FrameMassData 对象，失败返回 None
    
    Example:
        mass_data = get_frame_mass_data(model, "1")
        if mass_data:
            print(f"质量: {mass_data.mass_per_length}")
    """
    mass = get_frame_mass(model, frame_name)
    if mass is not None:
        return FrameMassData(frame_name=str(frame_name), mass_per_length=mass)
    return None


def delete_frame_mass(
    model,
    frame_name: str,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    删除杆件附加质量
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
        item_type: 操作范围
    
    Returns:
        0 表示成功
    
    Example:
        delete_frame_mass(model, "1")
    """
    return model.FrameObj.DeleteMass(str(frame_name), int(item_type))


def has_frame_mass(
    model,
    frame_name: str
) -> bool:
    """
    检查杆件是否有附加质量
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
    
    Returns:
        True=有附加质量, False=无附加质量
    
    Example:
        if has_frame_mass(model, "1"):
            print("杆件有附加质量")
    """
    mass = get_frame_mass(model, frame_name)
    return mass is not None and mass > 0



# -*- coding: utf-8 -*-
"""
utils - 工具函数

包含:
- deprecation: 弃用装饰器
- result: 统一返回值类型
- format_trans: 格式转换
"""

from .format_trans import *
from .deprecation import deprecated
from .result import Result, Ok, Err, BatchResult

__all__ = [
    "deprecated",
    "Result",
    "Ok", 
    "Err",
    "BatchResult",
]



# -*- coding: utf-8 -*-
"""
area_section.py - 面单元截面属性定义
对应 SAP2000 的 PropArea

本模块用于定义截面属性（是什么），而非分配截面到面单元（怎么用）。
截面分配功能请使用 types_for_areas 模块。

包含三种类型：
- Shell (壳): 薄壳、厚壳、薄板、厚板、膜、分层壳
- Plane (平面): 平面应力、平面应变
- Asolid (轴对称实体)

Usage:
    from section import AreaSection, AreaSectionType, ShellType
    
    # 获取面属性
    prop = AreaSection.get_by_name(model, "SLAB1")
    print(f"类型: {prop.prop_type}, 厚度: {prop.thickness}")
    
    # 创建壳截面
    shell = AreaSection(
        name="SLAB1",
        shell_type=ShellType.SHELL_THIN,
        material="4000Psi",
        thickness=0.2
    )
    shell._create(model)
"""

from dataclasses import dataclass, field
from typing import Optional, List, ClassVar
from enum import IntEnum


class AreaSectionType(IntEnum):
    """面属性类型 - GetNameList 的 PropType 参数"""
    ALL = 0
    SHELL = 1
    PLANE = 2
    ASOLID = 3


class ShellType(IntEnum):
    """壳类型 - GetShell_1 的 ShellType 参数"""
    SHELL_THIN = 1
    SHELL_THICK = 2
    PLATE_THIN = 3
    PLATE_THICK = 4
    MEMBRANE = 5
    SHELL_LAYERED = 6


class PlaneType(IntEnum):
    """平面类型 - GetPlane 的 MyType 参数"""
    PLANE_STRESS = 1
    PLANE_STRAIN = 2


@dataclass
class AreaModifiers:
    """面属性修正系数(10个)"""
    f11: float = 1.0
    f22: float = 1.0
    f12: float = 1.0
    m11: float = 1.0
    m22: float = 1.0
    m12: float = 1.0
    v13: float = 1.0
    v23: float = 1.0
    mass: float = 1.0
    weight: float = 1.0
    
    def to_list(self) -> List[float]:
        """转换为列表"""
        return [self.f11, self.f22, self.f12, self.m11, self.m22, 
                self.m12, self.v13, self.v23, self.mass, self.weight]
    
    @classmethod
    def from_list(cls, values: List[float]) -> 'AreaModifiers':
        """从列表创建"""
        if len(values) >= 10:
            return cls(
                f11=values[0], f22=values[1], f12=values[2],
                m11=values[3], m22=values[4], m12=values[5],
                v13=values[6], v23=values[7],
                mass=values[8], weight=values[9]
            )
        return cls()


@dataclass
class AreaSection:
    """面单元截面属性 - 对应 SAP2000 PropArea"""
    
    name: str = ""
    prop_type: Optional[AreaSectionType] = None
    shell_type: Optional[ShellType] = None
    plane_type: Optional[PlaneType] = None
    material: str = ""
    thickness: float = 0.0
    bending_thickness: float = 0.0
    material_angle: float = 0.0
    include_drilling_dof: bool = False
    incompatible_modes: bool = True
    arc_angle: float = 360.0
    color: int = -1
    notes: str = ""
    guid: Optional[str] = None
    _object_type: ClassVar[str] = "PropArea"

    @classmethod
    def get_by_name(cls, model, name: str) -> 'AreaSection':
        """获取指定名称的面属性"""
        prop = cls(name=name)
        prop._get(model)
        return prop
    
    @classmethod
    def get_all(cls, model, prop_type: AreaSectionType = AreaSectionType.ALL) -> List['AreaSection']:
        """获取所有面属性"""
        names = cls.get_name_list(model, prop_type)
        props = []
        for name in names:
            try:
                prop = cls.get_by_name(model, name)
                props.append(prop)
            except Exception:
                pass
        return props
    
    @staticmethod
    def get_count(model) -> int:
        """获取面属性总数"""
        return model.PropArea.Count()
    
    @staticmethod
    def get_name_list(model, prop_type: AreaSectionType = AreaSectionType.ALL) -> List[str]:
        """获取面属性名称列表"""
        result = model.PropArea.GetNameList(0, [], prop_type.value)
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            return list(result[1]) if result[1] else []
        return []
    
    def change_name(self, model, new_name: str) -> int:
        """修改属性名称"""
        ret = model.PropArea.ChangeName(self.name, new_name)
        if ret == 0:
            self.name = new_name
        return ret
    
    def get_modifiers(self, model) -> AreaModifiers:
        """获取修正系数"""
        result = model.PropArea.GetModifiers(self.name, [])
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            values = result[0]
            if values and len(values) >= 10:
                return AreaModifiers.from_list(list(values))
        return AreaModifiers()
    
    def set_modifiers(self, model, modifiers: AreaModifiers) -> int:
        """设置修正系数"""
        return model.PropArea.SetModifiers(self.name, modifiers.to_list())

    def _get(self, model) -> 'AreaSection':
        """从 SAP2000 获取面属性数据"""
        result = model.PropArea.GetTypeOAPI(self.name)
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            type_val = result[0]
            ret = result[1]
            if ret != 0:
                from exceptions import SectionError
                raise SectionError(f"面属性 {self.name} 不存在")
            try:
                self.prop_type = AreaSectionType(type_val)
            except ValueError:
                self.prop_type = AreaSectionType.SHELL
        else:
            from exceptions import SectionError
            raise SectionError(f"获取面属性 {self.name} 类型失败")
        
        if self.prop_type == AreaSectionType.SHELL:
            self._get_shell(model)
        elif self.prop_type == AreaSectionType.PLANE:
            self._get_plane(model)
        elif self.prop_type == AreaSectionType.ASOLID:
            self._get_asolid(model)
        return self
    
    def _get_shell(self, model):
        """获取壳属性数据"""
        result = model.PropArea.GetShell_1(self.name)
        if isinstance(result, (list, tuple)) and len(result) >= 10:
            try:
                self.shell_type = ShellType(result[0])
            except ValueError:
                self.shell_type = ShellType.SHELL_THIN
            self.include_drilling_dof = result[1]
            self.material = result[2] or ""
            self.material_angle = result[3]
            self.thickness = result[4]
            self.bending_thickness = result[5]
            self.color = result[6]
            self.notes = result[7] or ""
            self.guid = result[8] or None
    
    def _get_plane(self, model):
        """获取平面属性数据"""
        result = model.PropArea.GetPlane(self.name)
        if isinstance(result, (list, tuple)) and len(result) >= 9:
            try:
                self.plane_type = PlaneType(result[0])
            except ValueError:
                self.plane_type = PlaneType.PLANE_STRESS
            self.material = result[1] or ""
            self.material_angle = result[2]
            self.thickness = result[3]
            self.incompatible_modes = result[4]
            self.color = result[5]
            self.notes = result[6] or ""
            self.guid = result[7] or None
    
    def _get_asolid(self, model):
        """获取轴对称实体属性数据"""
        result = model.PropArea.GetAsolid(self.name)
        if isinstance(result, (list, tuple)) and len(result) >= 8:
            self.material = result[0] or ""
            self.material_angle = result[1]
            self.arc_angle = result[2]
            self.incompatible_modes = result[3]
            self.color = result[4]
            self.notes = result[5] or ""
            self.guid = result[6] or None

    def _create(self, model) -> int:
        """在 SAP2000 中创建面属性"""
        if self.shell_type is not None:
            return self._create_shell(model)
        elif self.plane_type is not None:
            return self._create_plane(model)
        elif self.prop_type == AreaSectionType.ASOLID:
            return self._create_asolid(model)
        else:
            self.shell_type = ShellType.SHELL_THIN
            return self._create_shell(model)
    
    def _create_shell(self, model) -> int:
        """创建壳属性"""
        return model.PropArea.SetShell_1(
            self.name,
            self.shell_type.value if self.shell_type else ShellType.SHELL_THIN.value,
            self.include_drilling_dof,
            self.material,
            self.material_angle,
            self.thickness,
            self.bending_thickness or self.thickness,
            self.color,
            self.notes,
            self.guid or ""
        )
    
    def _create_plane(self, model) -> int:
        """创建平面属性"""
        return model.PropArea.SetPlane(
            self.name,
            self.plane_type.value if self.plane_type else PlaneType.PLANE_STRESS.value,
            self.material,
            self.material_angle,
            self.thickness,
            self.incompatible_modes,
            self.color,
            self.notes,
            self.guid or ""
        )
    
    def _create_asolid(self, model) -> int:
        """创建轴对称实体属性"""
        return model.PropArea.SetAsolid(
            self.name,
            self.material,
            self.material_angle,
            self.arc_angle,
            self.incompatible_modes,
            self.color,
            self.notes,
            self.guid or ""
        )
    
    def _delete(self, model) -> int:
        """删除面属性"""
        return model.PropArea.Delete(self.name)
    
    def _update(self, model) -> int:
        """更新面属性"""
        return self._create(model)



# -*- coding: utf-8 -*-
"""
panel_zone.py - 节点域相关函数

用于设置梁柱节点的节点域 (Panel Zone)

SAP2000 API:
- PointObj.SetPanelZone / GetPanelZone / DeletePanelZone
"""

from typing import Optional
from .enums import (
    ItemType, 
    PanelZonePropType, 
    PanelZoneConnectivity, 
    PanelZoneLocalAxisFrom
)
from .data_classes import PanelZoneData


def set_point_panel_zone(
    model,
    point_name: str,
    prop_type: PanelZonePropType = PanelZonePropType.ELASTIC_FROM_COLUMN,
    thickness: float = 0.0,
    k1: float = 0.0,
    k2: float = 0.0,
    link_prop: str = "",
    connectivity: PanelZoneConnectivity = PanelZoneConnectivity.BEAMS_TO_OTHER,
    local_axis_from: PanelZoneLocalAxisFrom = PanelZoneLocalAxisFrom.FROM_COLUMN,
    local_axis_angle: float = 0.0,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    设置节点域
    
    节点域用于模拟梁柱节点区域的剪切变形。
    
    Args:
        model: SapModel 对象
        point_name: 节点名称
        prop_type: 属性类型
            - ELASTIC_FROM_COLUMN: 从柱截面计算弹性刚度
            - ELASTIC_FROM_COLUMN_DOUBLER: 从柱截面+加劲板计算
            - FROM_SPRING_STIFFNESS: 指定弹簧刚度
            - FROM_LINK_PROPERTY: 使用连接单元属性
        thickness: 加劲板厚度 [L] (仅 ELASTIC_FROM_COLUMN_DOUBLER 使用)
        k1: 弹簧刚度1 (仅 FROM_SPRING_STIFFNESS 使用)
        k2: 弹簧刚度2 (仅 FROM_SPRING_STIFFNESS 使用)
        link_prop: 连接单元属性名称 (仅 FROM_LINK_PROPERTY 使用)
        connectivity: 连接类型
        local_axis_from: 局部轴来源
        local_axis_angle: 局部轴角度 [deg]
        item_type: 项目类型
    
    Returns:
        0 表示成功
    
    Example:
        # 使用柱截面计算节点域刚度
        set_point_panel_zone(model, "5", PanelZonePropType.ELASTIC_FROM_COLUMN)
        
        # 使用加劲板
        set_point_panel_zone(
            model, "5", 
            PanelZonePropType.ELASTIC_FROM_COLUMN_DOUBLER,
            thickness=0.01  # 10mm 加劲板
        )
    """
    return model.PointObj.SetPanelZone(
        str(point_name),
        prop_type,
        thickness,
        k1,
        k2,
        link_prop,
        connectivity,
        local_axis_from,
        local_axis_angle,
        item_type
    )


def get_point_panel_zone(
    model,
    point_name: str
) -> Optional[PanelZoneData]:
    """
    获取节点域设置
    
    Args:
        model: SapModel 对象
        point_name: 节点名称
    
    Returns:
        PanelZoneData 对象，失败或无节点域返回 None
    
    Example:
        pz = get_point_panel_zone(model, "5")
        if pz:
            print(f"节点域类型: {pz.prop_type}")
    """
    try:
        result = model.PointObj.GetPanelZone(str(point_name))
        if isinstance(result, (list, tuple)) and len(result) >= 9:
            prop_type = result[0]
            thickness = result[1]
            k1 = result[2]
            k2 = result[3]
            link_prop = result[4]
            connectivity = result[5]
            local_axis_from = result[6]
            local_axis_angle = result[7]
            ret = result[8]
            
            if ret == 0:
                return PanelZoneData(
                    prop_type=PanelZonePropType(prop_type),
                    thickness=thickness,
                    k1=k1,
                    k2=k2,
                    link_prop=link_prop,
                    connectivity=PanelZoneConnectivity(connectivity),
                    local_axis_from=PanelZoneLocalAxisFrom(local_axis_from),
                    local_axis_angle=local_axis_angle
                )
    except Exception:
        pass
    return None


def delete_point_panel_zone(
    model,
    point_name: str,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    删除节点域
    
    Args:
        model: SapModel 对象
        point_name: 节点名称
        item_type: 项目类型
    
    Returns:
        0 表示成功
    
    Example:
        delete_point_panel_zone(model, "5")
    """
    return model.PointObj.DeletePanelZone(str(point_name), item_type)


def has_point_panel_zone(
    model,
    point_name: str
) -> bool:
    """
    检查节点是否有节点域
    
    Args:
        model: SapModel 对象
        point_name: 节点名称
    
    Returns:
        True=有节点域, False=没有
    """
    return get_point_panel_zone(model, point_name) is not None



# -*- coding: utf-8 -*-
"""
design/steel.py - 钢结构设计函数

SAP2000 DesignSteel API 的 Python 封装。
"""

from typing import List, Optional, Union

from .enums import SteelDesignCode, RatioType, ItemType, STEEL_CODE_NAMES, STEEL_CODE_FROM_NAME
from .data_classes import SteelSummaryResult, VerifyPassedResult


def get_steel_code(model) -> str:
    """获取当前钢结构设计规范
    
    Args:
        model: SapModel 对象
        
    Returns:
        规范名称字符串
    """
    result = model.DesignSteel.GetCode("")
    if isinstance(result, (list, tuple)) and len(result) >= 2:
        return result[0]
    return ""


def set_steel_code(model, code: Union[SteelDesignCode, str]) -> int:
    """设置钢结构设计规范
    
    Args:
        model: SapModel 对象
        code: 规范枚举或规范名称字符串
        
    Returns:
        0 表示成功，非 0 表示失败
    """
    if isinstance(code, SteelDesignCode):
        code_name = STEEL_CODE_NAMES.get(code, "AISC 360-10")
    else:
        code_name = code
    
    ret = model.DesignSteel.SetCode(code_name)
    if isinstance(ret, (list, tuple)):
        return ret[-1]
    return ret


def start_steel_design(model) -> int:
    """开始钢结构设计
    
    注意：需要先运行分析，且模型中存在钢框架对象。
    
    Args:
        model: SapModel 对象
        
    Returns:
        0 表示成功，非 0 表示失败
    """
    ret = model.DesignSteel.StartDesign()
    if isinstance(ret, (list, tuple)):
        return ret[-1]
    return ret


def delete_steel_results(model) -> int:
    """删除所有钢结构设计结果
    
    Args:
        model: SapModel 对象
        
    Returns:
        0 表示成功，非 0 表示失败
    """
    ret = model.DesignSteel.DeleteResults()
    if isinstance(ret, (list, tuple)):
        return ret[-1]
    return ret


def get_steel_results_available(model) -> bool:
    """检查钢结构设计结果是否可用
    
    Args:
        model: SapModel 对象
        
    Returns:
        True 表示结果可用，False 表示不可用
    """
    result = model.DesignSteel.GetResultsAvailable()
    if isinstance(result, (list, tuple)):
        return bool(result[0])
    return bool(result)


def get_steel_summary_results(
    model,
    name: str,
    item_type: ItemType = ItemType.OBJECT
) -> List[SteelSummaryResult]:
    """获取钢结构设计汇总结果
    
    Args:
        model: SapModel 对象
        name: 对象名称、组名称或忽略（取决于 item_type）
        item_type: 对象选择类型
        
    Returns:
        设计结果列表
    """
    result = model.DesignSteel.GetSummaryResults(
        name, 0, [], [], [], [], [], [], [], int(item_type)
    )
    
    results = []
    if isinstance(result, (list, tuple)) and len(result) >= 9:
        num_items = result[0]
        frame_names = result[1] if result[1] else []
        ratios = result[2] if result[2] else []
        ratio_types = result[3] if result[3] else []
        locations = result[4] if result[4] else []
        combo_names = result[5] if result[5] else []
        error_summaries = result[6] if result[6] else []
        warning_summaries = result[7] if result[7] else []
        
        for i in range(num_items):
            # 安全处理 ratio_type，未知值默认为 NONE
            try:
                ratio_type_val = ratio_types[i] if i < len(ratio_types) else 0
                ratio_type = RatioType(ratio_type_val)
            except ValueError:
                ratio_type = RatioType.NONE
            
            results.append(SteelSummaryResult(
                frame_name=frame_names[i] if i < len(frame_names) else "",
                ratio=ratios[i] if i < len(ratios) else 0.0,
                ratio_type=ratio_type,
                location=locations[i] if i < len(locations) else 0.0,
                combo_name=combo_names[i] if i < len(combo_names) else "",
                error_summary=error_summaries[i] if i < len(error_summaries) else "",
                warning_summary=warning_summaries[i] if i < len(warning_summaries) else "",
            ))
    
    return results


def get_steel_design_group(model) -> List[str]:
    """获取选中用于钢结构设计的组
    
    Args:
        model: SapModel 对象
        
    Returns:
        组名称列表
    """
    result = model.DesignSteel.GetGroup(0, [])
    if isinstance(result, (list, tuple)) and len(result) >= 2:
        names = result[1]
        if names:
            return list(names)
    return []


def set_steel_design_group(model, name: str, selected: bool = True) -> int:
    """设置组是否用于钢结构设计
    
    Args:
        model: SapModel 对象
        name: 组名称
        selected: True 选中，False 取消选中
        
    Returns:
        0 表示成功，非 0 表示失败
    """
    ret = model.DesignSteel.SetGroup(name, selected)
    if isinstance(ret, (list, tuple)):
        return ret[-1]
    return ret


def get_steel_design_section(model, name: str) -> str:
    """获取框架对象的设计截面
    
    Args:
        model: SapModel 对象
        name: 框架对象名称
        
    Returns:
        设计截面名称
    """
    result = model.DesignSteel.GetDesignSection(name, "")
    if isinstance(result, (list, tuple)) and len(result) >= 2:
        return result[0]
    return ""


def set_steel_design_section(
    model,
    name: str,
    prop_name: str = "",
    last_analysis: bool = False,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """设置框架对象的设计截面
    
    Args:
        model: SapModel 对象
        name: 对象名称、组名称或忽略（取决于 item_type）
        prop_name: 截面名称（last_analysis=False 时使用）
        last_analysis: True 使用最后分析截面，False 使用指定截面
        item_type: 对象选择类型
        
    Returns:
        0 表示成功，非 0 表示失败
    """
    ret = model.DesignSteel.SetDesignSection(name, prop_name, last_analysis, int(item_type))
    if isinstance(ret, (list, tuple)):
        return ret[-1]
    return ret


def get_steel_combo_strength(model) -> List[str]:
    """获取用于强度设计的荷载组合
    
    Args:
        model: SapModel 对象
        
    Returns:
        组合名称列表
    """
    result = model.DesignSteel.GetComboStrength(0, [])
    if isinstance(result, (list, tuple)) and len(result) >= 2:
        names = result[1]
        if names:
            return list(names)
    return []


def set_steel_combo_strength(model, name: str, selected: bool = True) -> int:
    """设置荷载组合是否用于强度设计
    
    Args:
        model: SapModel 对象
        name: 荷载组合名称
        selected: True 选中，False 取消选中
        
    Returns:
        0 表示成功，非 0 表示失败
    """
    ret = model.DesignSteel.SetComboStrength(name, selected)
    if isinstance(ret, (list, tuple)):
        return ret[-1]
    return ret


def get_steel_combo_deflection(model) -> List[str]:
    """获取用于挠度设计的荷载组合
    
    Args:
        model: SapModel 对象
        
    Returns:
        组合名称列表
    """
    result = model.DesignSteel.GetComboDeflection(0, [])
    if isinstance(result, (list, tuple)) and len(result) >= 2:
        names = result[1]
        if names:
            return list(names)
    return []


def set_steel_combo_deflection(model, name: str, selected: bool = True) -> int:
    """设置荷载组合是否用于挠度设计
    
    Args:
        model: SapModel 对象
        name: 荷载组合名称
        selected: True 选中，False 取消选中
        
    Returns:
        0 表示成功，非 0 表示失败
    """
    ret = model.DesignSteel.SetComboDeflection(name, selected)
    if isinstance(ret, (list, tuple)):
        return ret[-1]
    return ret


def reset_steel_overwrites(model) -> int:
    """重置所有钢结构设计覆盖为默认值
    
    Args:
        model: SapModel 对象
        
    Returns:
        0 表示成功，非 0 表示失败
    """
    ret = model.DesignSteel.ResetOverwrites()
    if isinstance(ret, (list, tuple)):
        return ret[-1]
    return ret


def verify_steel_passed(model) -> VerifyPassedResult:
    """验证钢结构设计是否通过
    
    获取未通过设计检查或尚未检查的框架对象。
    
    Args:
        model: SapModel 对象
        
    Returns:
        验证结果
    """
    result = model.DesignSteel.VerifyPassed(0, 0, 0, [])
    
    if isinstance(result, (list, tuple)) and len(result) >= 4:
        total_count = result[0]
        failed_count = result[1]
        unchecked_count = result[2]
        names = result[3] if result[3] else []
        
        return VerifyPassedResult(
            total_count=total_count,
            failed_count=failed_count,
            unchecked_count=unchecked_count,
            frame_names=list(names) if names else [],
        )
    
    return VerifyPassedResult(0, 0, 0, [])


def verify_steel_sections(model) -> List[str]:
    """验证分析截面与设计截面是否一致
    
    获取分析截面与设计截面不同的框架对象。
    
    Args:
        model: SapModel 对象
        
    Returns:
        截面不一致的框架对象名称列表
    """
    result = model.DesignSteel.VerifySections(0, [])
    
    if isinstance(result, (list, tuple)) and len(result) >= 2:
        names = result[1]
        if names:
            return list(names)
    return []



# -*- coding: utf-8 -*-
"""
约束操作工具 - 刚性隔板、节点约束

重构: 复用 PySap2000.constraints 和 PySap2000.point 模块
"""

from langchain.tools import tool

from .base import get_sap_model, to_json, success_response, error_response, safe_sap_call

# 导入 PySap2000 封装
from PySap2000.point import set_point_constraint


@tool
@safe_sap_call
def get_constraint_list() -> str:
    """获取模型中所有约束的名称列表。"""
    model = get_sap_model()
    ret = model.ConstraintDef.GetNameList(0, [])
    
    if isinstance(ret, (list, tuple)) and len(ret) >= 2:
        constraints = list(ret[1]) if ret[1] else []
        return to_json({"约束数": len(constraints), "约束列表": constraints})
    return to_json({"约束列表": []})


@tool
@safe_sap_call
def create_diaphragm_constraint(name: str, axis: str = "Z") -> str:
    """
    创建刚性隔板约束。
    
    Args:
        name: 约束名称
        axis: 约束轴向 (X, Y, Z)，默认 Z
    """
    model = get_sap_model()
    model.SetModelIsLocked(False)
    
    axis_map = {"X": 1, "Y": 2, "Z": 3}
    axis_code = axis_map.get(axis.upper(), 3)
    
    ret = model.ConstraintDef.SetDiaphragm(name, axis_code)
    
    if ret == 0:
        return success_response(f"刚性隔板约束 '{name}' 创建成功", 轴向=axis)
    return error_response("创建约束失败")


@tool
@safe_sap_call
def assign_point_constraint(point_name: str, constraint_name: str) -> str:
    """
    将约束分配给节点。
    
    Args:
        point_name: 节点名称
        constraint_name: 约束名称
    """
    model = get_sap_model()
    model.SetModelIsLocked(False)
    
    # 使用 PySap2000 封装
    ret = set_point_constraint(model, point_name, constraint_name)
    
    if ret == 0:
        return success_response(f"节点 '{point_name}' 已分配约束 '{constraint_name}'")
    return error_response("分配约束失败")



# -*- coding: utf-8 -*-
"""
mass.py - Cable 质量指派

SAP2000 API:
- CableObj.SetMass(Name, MassOverL, Replace, ItemType)
- CableObj.GetMass(Name, MassOverL)
- CableObj.DeleteMass(Name, ItemType)
"""

from typing import Optional
from .modifier import CableItemType


def set_cable_mass(
    model,
    cable_name: str,
    mass_per_length: float,
    replace: bool = False,
    item_type: CableItemType = CableItemType.OBJECT
) -> int:
    """
    设置 Cable 附加质量
    
    Args:
        model: SapModel 对象
        cable_name: Cable 名称
        mass_per_length: 单位长度质量 [M/L]
        replace: True=替换现有质量, False=叠加
        item_type: 操作范围
    
    Returns:
        0 表示成功
    
    Example:
        set_cable_mass(model, "1", 0.001)
    """
    return model.CableObj.SetMass(str(cable_name), mass_per_length, replace, int(item_type))


def get_cable_mass(model, cable_name: str) -> Optional[float]:
    """
    获取 Cable 附加质量
    
    Args:
        model: SapModel 对象
        cable_name: Cable 名称
    
    Returns:
        单位长度质量 [M/L]，失败返回 None
    
    Example:
        mass = get_cable_mass(model, "1")
    """
    try:
        result = model.CableObj.GetMass(str(cable_name), 0.0)
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            return result[0]
    except Exception:
        pass
    return None


def delete_cable_mass(
    model,
    cable_name: str,
    item_type: CableItemType = CableItemType.OBJECT
) -> int:
    """
    删除 Cable 附加质量
    
    Args:
        model: SapModel 对象
        cable_name: Cable 名称
        item_type: 操作范围
    
    Returns:
        0 表示成功
    """
    return model.CableObj.DeleteMass(str(cable_name), int(item_type))



# -*- coding: utf-8 -*-
"""
enums.py - 分析相关枚举

SAP2000 Analyze API 使用的枚举类型
"""

from enum import IntEnum


class CaseStatus(IntEnum):
    """
    工况分析状态
    
    对应 Analyze.GetCaseStatus 返回的 Status 值
    """
    NOT_RUN = 1           # 未运行
    COULD_NOT_START = 2   # 无法启动
    NOT_FINISHED = 3      # 未完成
    FINISHED = 4          # 已完成


class SolverType(IntEnum):
    """
    求解器类型
    
    对应 Analyze.GetSolverOption_3 / SetSolverOption_3 的 SolverType 参数
    """
    STANDARD = 0          # 标准求解器
    ADVANCED = 1          # 高级求解器
    MULTI_THREADED = 2    # 多线程求解器


class SolverProcessType(IntEnum):
    """
    求解器进程类型
    
    对应 Analyze.GetSolverOption_3 / SetSolverOption_3 的 SolverProcessType 参数
    """
    AUTO = 0              # 自动 (程序决定)
    GUI_PROCESS = 1       # GUI 进程
    SEPARATE_PROCESS = 2  # 独立进程



# -*- coding: utf-8 -*-
"""
modifier.py - 杆件截面修改器相关函数

用于设置杆件的截面属性修改器（刚度折减等）

SAP2000 API:
- FrameObj.SetModifiers(Name, Value[], ItemType)
- FrameObj.GetModifiers(Name, Value[])
- FrameObj.DeleteModifiers(Name, ItemType)
"""

from typing import Tuple, Optional
from .enums import ItemType
from .data_classes import FrameModifierData


def set_frame_modifiers(
    model,
    frame_name: str,
    area: float = 1.0,
    shear_2: float = 1.0,
    shear_3: float = 1.0,
    torsion: float = 1.0,
    inertia_22: float = 1.0,
    inertia_33: float = 1.0,
    mass: float = 1.0,
    weight: float = 1.0,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    设置杆件截面修改器
    
    修改器用于调整截面属性，常用于刚度折减。
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
        area: 截面面积修改器 (A)
        shear_2: 局部2方向剪切面积修改器 (As2)
        shear_3: 局部3方向剪切面积修改器 (As3)
        torsion: 扭转常数修改器 (J)
        inertia_22: 局部2轴惯性矩修改器 (I22)
        inertia_33: 局部3轴惯性矩修改器 (I33)
        mass: 质量修改器
        weight: 重量修改器
        item_type: 操作范围
    
    Returns:
        0 表示成功
    
    Example:
        # 设置 I33 修改器为 0.5 (刚度折减50%)
        set_frame_modifiers(model, "1", inertia_33=0.5)
        
        # 梁刚度折减 (I22=0.4, I33=0.4)
        set_frame_modifiers(model, "1", inertia_22=0.4, inertia_33=0.4)
        
        # 柱刚度折减 (A=0.7, I22=0.7, I33=0.7)
        set_frame_modifiers(model, "1", area=0.7, inertia_22=0.7, inertia_33=0.7)
    """
    modifiers = [area, shear_2, shear_3, torsion, inertia_22, inertia_33, mass, weight]
    result = model.FrameObj.SetModifiers(str(frame_name), modifiers, int(item_type))
    # 解析返回值
    if isinstance(result, (list, tuple)) and len(result) >= 2:
        return result[-1]
    return result


def set_frame_modifiers_tuple(
    model,
    frame_name: str,
    modifiers: Tuple[float, ...],
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    设置杆件截面修改器（元组格式）
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
        modifiers: 8个修改器值的元组
            (A, As2, As3, J, I22, I33, Mass, Weight)
        item_type: 操作范围
    
    Returns:
        0 表示成功
    
    Example:
        set_frame_modifiers_tuple(model, "1", (1, 1, 1, 1, 1, 0.5, 1, 1))
    """
    m_list = list(modifiers)
    while len(m_list) < 8:
        m_list.append(1.0)
    return model.FrameObj.SetModifiers(str(frame_name), m_list[:8], int(item_type))


def get_frame_modifiers(
    model,
    frame_name: str
) -> Optional[FrameModifierData]:
    """
    获取杆件截面修改器
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
    
    Returns:
        FrameModifierData 对象，失败返回 None
    
    Example:
        modifiers = get_frame_modifiers(model, "1")
        if modifiers:
            print(f"I33修改器: {modifiers.inertia_33}")
    """
    try:
        result = model.FrameObj.GetModifiers(str(frame_name), [0.0] * 8)
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            values = result[0]
            if values and len(values) >= 8:
                return FrameModifierData.from_tuple(str(frame_name), tuple(values))
    except Exception:
        pass
    return None


def get_frame_modifiers_tuple(
    model,
    frame_name: str
) -> Optional[Tuple[float, ...]]:
    """
    获取杆件截面修改器（元组格式）
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
    
    Returns:
        8个修改器值的元组，失败返回 None
    
    Example:
        modifiers = get_frame_modifiers_tuple(model, "1")
        if modifiers:
            print(f"修改器: {modifiers}")
    """
    try:
        result = model.FrameObj.GetModifiers(str(frame_name), [0.0] * 8)
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            values = result[0]
            if values:
                return tuple(values)
    except Exception:
        pass
    return None


def delete_frame_modifiers(
    model,
    frame_name: str,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    删除杆件修改器（恢复默认值1.0）
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
        item_type: 操作范围
    
    Returns:
        0 表示成功
    
    Example:
        delete_frame_modifiers(model, "1")
    """
    return model.FrameObj.DeleteModifiers(str(frame_name), int(item_type))



# -*- coding: utf-8 -*-
"""
视图操作工具 - 刷新视图、缩放

重构: 复用 PySap2000.view 模块（如果存在）
"""

from langchain.tools import tool

from .base import get_sap_model, success_response, error_response, safe_sap_call


@tool
@safe_sap_call
def refresh_view() -> str:
    """刷新 SAP2000 视图。"""
    model = get_sap_model()
    model.View.RefreshView(0, False)
    return success_response("视图已刷新")


@tool
@safe_sap_call
def zoom_all() -> str:
    """缩放视图以显示所有对象。"""
    model = get_sap_model()
    ret = model.View.RefreshView(0, True)
    
    if ret == 0:
        return success_response("已缩放至全部")
    return error_response("缩放失败")



# -*- coding: utf-8 -*-
"""
PySap2000 可视化模块

用于将 SAP2000 模型导出为可视化格式（需要完整几何信息）
"""

# 延迟导入，避免在 Rhino 环境中导入失败
__all__ = [
    'WebExporter',
    'WebExporterSolid',
    'MeshGenerator',
    'RhinoBuilder',
]

def __getattr__(name):
    if name == 'WebExporter':
        from .web import WebExporter
        return WebExporter
    elif name == 'WebExporterSolid':
        from .web import WebExporterSolid
        return WebExporterSolid
    elif name == 'MeshGenerator':
        from .web import MeshGenerator
        return MeshGenerator
    elif name == 'RhinoBuilder':
        from .rhino import RhinoBuilder
        return RhinoBuilder
    raise AttributeError(f"module '{__name__}' has no attribute '{name}'")



# -*- coding: utf-8 -*-
"""
测试配置文件读取
"""

import os
import sys

# 添加路径
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

def test_config():
    """测试配置读取"""
    print("=" * 60)
    print("  测试配置文件读取")
    print("=" * 60)
    print()
    
    # 导入 load_config
    from web_agent.sap_agent import load_config
    
    config = load_config()
    
    print(f"配置文件路径: {config['config_file']}")
    print(f"API Key: {config['api_key'][:10]}...{config['api_key'][-10:] if config['api_key'] else '(未设置)'}")
    print(f"提供商: {config['provider']}")
    print(f"模型: {config['model_name']}")
    print()
    
    if not config['api_key']:
        print("❌ API Key 未设置")
        print(f"请编辑配置文件: {config['config_file']}")
        return False
    
    print("✓ 配置读取成功")
    print()
    
    # 测试 Agent 初始化
    print("测试 Agent 初始化...")
    
    # 先检查 LangChain 是否可用
    try:
        from langchain_agent import LANGCHAIN_AVAILABLE
        if not LANGCHAIN_AVAILABLE:
            print("❌ LangChain 未安装")
            print("请运行: pip install -r PySap2000/web_agent/requirements.txt")
            return False
    except ImportError:
        print("❌ langchain_agent 模块未找到")
        print("请运行: pip install -r PySap2000/web_agent/requirements.txt")
        return False
    
    from web_agent.sap_agent import get_or_create_agent
    
    agent = get_or_create_agent()
    
    if agent is None:
        print("❌ Agent 初始化失败")
        return False
    
    print("✓ Agent 初始化成功")
    print()
    
    # 测试简单对话（不需要 SAP2000）
    print("测试对话功能...")
    try:
        # 清空历史
        agent.clear_history()
        
        response = agent.chat("中国的首都是哪里？请简短回答。")
        print(f"问题: 中国的首都是哪里？")
        print(f"回答: {response}")
        print()
        print("✓ 对话功能正常")
        return True
    except Exception as e:
        print(f"❌ 对话失败: {e}")
        import traceback
        traceback.print_exc()
        return False


if __name__ == "__main__":
    try:
        success = test_config()
        if success:
            print()
            print("=" * 60)
            print("  所有测试通过！")
            print("=" * 60)
        else:
            print()
            print("=" * 60)
            print("  测试失败，请检查配置")
            print("=" * 60)
    except Exception as e:
        print(f"\n错误: {e}")
        import traceback
        traceback.print_exc()
    
    input("\n按 Enter 键退出...")



"""
SapAgent 一键安装器
- 自动检测环境
- 自动注册协议
- 自动配置开机启动（可选）
- 提供友好的图形界面
"""
import sys
import os
import winreg
import tkinter as tk
from tkinter import ttk, messagebox
from pathlib import Path
import subprocess


class EasyInstaller:
    def __init__(self):
        self.root = tk.Tk()
        self.root.title("SapAgent 一键安装")
        self.root.geometry("500x400")
        self.root.resizable(False, False)
        
        # 获取 exe 路径
        if getattr(sys, 'frozen', False):
            self.exe_path = sys.executable
            self.exe_dir = Path(sys.executable).parent
        else:
            self.exe_path = str(Path(__file__).parent / "SapAgent.exe")
            self.exe_dir = Path(__file__).parent
        
        self.setup_ui()
    
    def setup_ui(self):
        """设置界面"""
        # 标题
        title = tk.Label(
            self.root,
            text="SapAgent 一键安装",
            font=("Arial", 16, "bold")
        )
        title.pack(pady=20)
        
        # 说明
        desc = tk.Label(
            self.root,
            text="让 SAP2000 连接到 www.spancore.cn 在线工具",
            font=("Arial", 10)
        )
        desc.pack(pady=5)
        
        # 分隔线
        ttk.Separator(self.root, orient='horizontal').pack(fill='x', pady=20)
        
        # 检查项
        self.check_frame = tk.Frame(self.root)
        self.check_frame.pack(pady=10)
        
        self.checks = {
            'sap2000': {'label': '检查 SAP2000 安装', 'status': '⏳ 检查中...'},
            'com': {'label': '检查 COM 组件注册', 'status': '⏳ 检查中...'},
            'protocol': {'label': '注册 sapagent:// 协议', 'status': '⏳ 待处理'},
        }
        
        self.check_labels = {}
        for key, info in self.checks.items():
            frame = tk.Frame(self.check_frame)
            frame.pack(fill='x', pady=5)
            
            tk.Label(frame, text=info['label'], width=25, anchor='w').pack(side='left')
            status_label = tk.Label(frame, text=info['status'], width=20, anchor='w')
            status_label.pack(side='left')
            self.check_labels[key] = status_label
        
        # 可选项
        self.auto_start_var = tk.BooleanVar(value=False)
        auto_start_check = tk.Checkbutton(
            self.root,
            text="开机自动启动 SapAgent（推荐）",
            variable=self.auto_start_var
        )
        auto_start_check.pack(pady=10)
        
        # 按钮
        button_frame = tk.Frame(self.root)
        button_frame.pack(pady=20)
        
        self.install_btn = tk.Button(
            button_frame,
            text="开始安装",
            command=self.start_install,
            width=15,
            height=2,
            bg="#4CAF50",
            fg="white",
            font=("Arial", 10, "bold")
        )
        self.install_btn.pack(side='left', padx=5)
        
        tk.Button(
            button_frame,
            text="取消",
            command=self.root.quit,
            width=15,
            height=2
        ).pack(side='left', padx=5)
        
        # 状态栏
        self.status_label = tk.Label(
            self.root,
            text="准备就绪",
            relief=tk.SUNKEN,
            anchor='w'
        )
        self.status_label.pack(side='bottom', fill='x')
        
        # 自动检查
        self.root.after(500, self.check_environment)
    
    def update_status(self, key, status, color='black'):
        """更新检查状态"""
        self.check_labels[key].config(text=status, fg=color)
        self.root.update()
    
    def check_environment(self):
        """检查环境"""
        self.status_label.config(text="正在检查环境...")
        
        # 检查 SAP2000
        try:
            key = winreg.OpenKey(
                winreg.HKEY_CLASSES_ROOT,
                "CSI.SAP2000.API.SapObject",
                0,
                winreg.KEY_READ
            )
            winreg.CloseKey(key)
            self.update_status('sap2000', '✓ 已安装', 'green')
            self.update_status('com', '✓ 已注册', 'green')
        except WindowsError:
            self.update_status('sap2000', '✓ 已安装', 'green')
            self.update_status('com', '✗ 未注册', 'red')
        
        self.update_status('protocol', '⏳ 待安装', 'orange')
        self.status_label.config(text="环境检查完成")
    
    def start_install(self):
        """开始安装"""
        self.install_btn.config(state='disabled')
        self.status_label.config(text="正在安装...")
        
        try:
            # 1. 注册协议
            self.update_status('protocol', '⏳ 注册中...', 'orange')
            if self.register_protocol():
                self.update_status('protocol', '✓ 已注册', 'green')
            else:
                self.update_status('protocol', '✗ 注册失败', 'red')
                raise Exception("协议注册失败")
            
            # 2. 配置开机启动
            if self.auto_start_var.get():
                self.setup_auto_start()
            
            # 3. 完成
            self.status_label.config(text="安装完成！")
            messagebox.showinfo(
                "安装成功",
                "SapAgent 已成功安装！\n\n"
                "现在可以访问 www.spancore.cn 使用在线工具了。\n\n"
                "提示：打开 SAP2000 和模型后，工具会自动连接。"
            )
            self.root.quit()
            
        except Exception as e:
            self.status_label.config(text=f"安装失败: {e}")
            messagebox.showerror("安装失败", str(e))
            self.install_btn.config(state='normal')
    
    def register_protocol(self):
        """注册 sapagent:// 协议"""
        try:
            protocol_name = "sapagent"
            
            # 创建协议键
            key = winreg.CreateKey(
                winreg.HKEY_CURRENT_USER,
                f"Software\\Classes\\{protocol_name}"
            )
            winreg.SetValueEx(key, "", 0, winreg.REG_SZ, "URL:SapAgent Protocol")
            winreg.SetValueEx(key, "URL Protocol", 0, winreg.REG_SZ, "")
            winreg.CloseKey(key)
            
            # 创建命令键
            key = winreg.CreateKey(
                winreg.HKEY_CURRENT_USER,
                f"Software\\Classes\\{protocol_name}\\shell\\open\\command"
            )
            winreg.SetValueEx(key, "", 0, winreg.REG_SZ, f'"{self.exe_path}" "%1"')
            winreg.CloseKey(key)
            
            return True
        except Exception as e:
            print(f"注册协议失败: {e}")
            return False
    
    def setup_auto_start(self):
        """配置开机启动"""
        try:
            key = winreg.OpenKey(
                winreg.HKEY_CURRENT_USER,
                r"Software\Microsoft\Windows\CurrentVersion\Run",
                0,
                winreg.KEY_SET_VALUE
            )
            winreg.SetValueEx(
                key,
                "SapAgent",
                0,
                winreg.REG_SZ,
                f'"{self.exe_path}"'
            )
            winreg.CloseKey(key)
        except Exception as e:
            print(f"配置开机启动失败: {e}")
    
    def run(self):
        """运行安装器"""
        self.root.mainloop()


def main():
    """主函数"""
    app = EasyInstaller()
    app.run()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
global_parameters - 全局参数模块
包含单位系统、项目信息、自由度设置等全局参数
"""

from .units import Units, UnitSystem
from .project_info import ProjectInfo
from .model_settings import ModelSettings, ActiveDOF

__all__ = [
    'Units',
    'UnitSystem',
    'ProjectInfo',
    'ModelSettings',
    'ActiveDOF',
]



# -*- coding: utf-8 -*-
"""
local_axes.py - 杆件局部坐标轴相关函数

用于设置杆件的局部坐标轴方向

SAP2000 API:
- FrameObj.SetLocalAxes(Name, Ang, ItemType)
- FrameObj.GetLocalAxes(Name, Ang, Advanced)
- FrameObj.GetTransformationMatrix(Name, Value[], IsGlobal)
"""

from typing import Tuple, Optional, List
from .enums import ItemType
from .data_classes import FrameLocalAxesData


def set_frame_local_axes(
    model,
    frame_name: str,
    angle: float,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    设置杆件局部轴角度
    
    局部2和3轴绕正局部1轴旋转的角度。
    正角度从局部+1轴方向看为逆时针。
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
        angle: 旋转角度 [deg]
        item_type: 操作范围
    
    Returns:
        0 表示成功
    
    Example:
        # 旋转局部轴 30 度
        set_frame_local_axes(model, "1", 30)
        
        # 旋转局部轴 90 度 (常用于斜撑)
        set_frame_local_axes(model, "B1", 90)
    """
    return model.FrameObj.SetLocalAxes(str(frame_name), angle, int(item_type))


def get_frame_local_axes(
    model,
    frame_name: str
) -> Optional[FrameLocalAxesData]:
    """
    获取杆件局部轴角度
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
    
    Returns:
        FrameLocalAxesData 对象，失败返回 None
    
    Example:
        axes = get_frame_local_axes(model, "1")
        if axes:
            print(f"局部轴角度: {axes.angle}°")
    """
    try:
        result = model.FrameObj.GetLocalAxes(str(frame_name))
        if isinstance(result, (list, tuple)) and len(result) >= 3:
            return FrameLocalAxesData(
                frame_name=str(frame_name),
                angle=result[0],
                advanced=result[1]
            )
    except Exception:
        pass
    return None


def get_frame_transformation_matrix(
    model,
    frame_name: str,
    is_global: bool = True
) -> Optional[List[float]]:
    """
    获取杆件变换矩阵
    
    返回 3x3 变换矩阵（9个值），用于局部坐标和全局坐标转换。
    
    Args:
        model: SapModel 对象
        frame_name: 杆件名称
        is_global: True=全局坐标系, False=当前坐标系
    
    Returns:
        9个浮点数的列表 (3x3矩阵按行排列)，失败返回 None
    
    Example:
        matrix = get_frame_transformation_matrix(model, "1")
        if matrix:
            # matrix[0:3] = 局部1轴在全局坐标系中的方向
            # matrix[3:6] = 局部2轴在全局坐标系中的方向
            # matrix[6:9] = 局部3轴在全局坐标系中的方向
            print(f"局部1轴方向: {matrix[0:3]}")
    """
    try:
        result = model.FrameObj.GetTransformationMatrix(
            str(frame_name), [0.0] * 12, is_global
        )
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            values = result[0]
            if values and len(values) >= 9:
                return list(values[:9])
    except Exception:
        pass
    return None



# -*- coding: utf-8 -*-
"""
design/chinese_2010.py - 中国钢结构规范 GB 50017-2010 设计函数

SAP2000 DesignSteel.Chinese_2010 API 的 Python 封装。
"""

from dataclasses import dataclass
from enum import IntEnum
from typing import Optional, Tuple, Union

from .enums import ItemType


class FramingType(IntEnum):
    """结构体系类型"""
    PROGRAM_DEFAULT = 0     # 按首选项设置
    SMF = 1                 # 有侧移弯矩框架
    CBF = 2                 # 中心支撑框架
    EBF = 3                 # 偏心支撑框架
    NMF = 4                 # 无侧移弯矩框架


class ElementType(IntEnum):
    """构件类型"""
    PROGRAM_DETERMINED = 0  # 程序确定
    COLUMN = 1              # 柱
    BEAM = 2                # 梁
    BRACE = 3               # 支撑
    TRUSS = 4               # 桁架


class SeismicDesignGrade(IntEnum):
    """抗震设计等级"""
    GRADE_I = 1             # 一级
    GRADE_II = 2            # 二级
    GRADE_III = 3           # 三级
    GRADE_IV = 4            # 四级
    NON_SEISMIC = 5         # 非抗震


class MultiResponseDesign(IntEnum):
    """多工况设计方法"""
    ENVELOPES = 1           # 包络
    STEP_BY_STEP = 2        # 逐步
    LAST_STEP = 3           # 最后一步
    ENVELOPES_ALL = 4       # 包络-全部
    STEP_BY_STEP_ALL = 5    # 逐步-全部


class DeflectionCheckType(IntEnum):
    """挠度检查类型"""
    PROGRAM_DEFAULT = 0     # 程序默认
    RATIO = 1               # 相对值
    ABSOLUTE = 2            # 绝对值
    BOTH = 3                # 两者


class OverwriteItem(IntEnum):
    """覆盖项编号 (1-51)"""
    FRAMING_TYPE = 1                    # 结构体系类型
    ELEMENT_TYPE = 2                    # 构件类型
    IS_TRANSFER_COLUMN = 3              # 是否转换柱
    SEISMIC_MAGNIFICATION = 4           # 地震放大系数
    IS_ROLLED_SECTION = 5               # 是否轧制截面
    IS_FLANGE_CUT_BY_GAS = 6            # 翼缘是否气割
    IS_BOTH_END_PINNED = 7              # 两端是否铰接
    IGNORE_BT_CHECK = 8                 # 忽略宽厚比检查
    CLASSIFY_AS_FLEXO_COMPRESSION = 9   # 按压弯构件分类
    IS_BEAM_TOP_LOADED = 10             # 梁顶部是否受荷
    CONSIDER_DEFLECTION = 11            # 考虑挠度
    DEFLECTION_CHECK_TYPE = 12          # 挠度检查类型
    DL_DEFLECTION_RATIO = 13            # 恒载挠度限值 L/Value
    SDL_LL_DEFLECTION_RATIO = 14        # 附加恒载+活载挠度限值 L/Value
    LL_DEFLECTION_RATIO = 15            # 活载挠度限值 L/Value
    TOTAL_DEFLECTION_RATIO = 16         # 总荷载挠度限值 L/Value
    TOTAL_CAMBER_RATIO = 17             # 总起拱限值 L/Value
    DL_DEFLECTION_ABS = 18              # 恒载挠度限值 绝对值
    SDL_LL_DEFLECTION_ABS = 19          # 附加恒载+活载挠度限值 绝对值
    LL_DEFLECTION_ABS = 20              # 活载挠度限值 绝对值
    TOTAL_DEFLECTION_ABS = 21           # 总荷载挠度限值 绝对值
    TOTAL_CAMBER_ABS = 22               # 总起拱限值 绝对值
    SPECIFIED_CAMBER = 23               # 指定起拱值
    NET_AREA_RATIO = 24                 # 净面积与总面积比
    LIVE_LOAD_REDUCTION = 25            # 活载折减系数
    UNBRACED_RATIO_MAJOR = 26           # 主轴无支撑长度比
    UNBRACED_RATIO_MINOR_LTB = 27       # 次轴侧扭屈曲无支撑长度比
    MUE_MAJOR = 28                      # 主轴计算长度系数 μ
    MUE_MINOR = 29                      # 次轴计算长度系数 μ
    BETA_M_MAJOR = 30                   # 主轴等效弯矩系数 βm
    BETA_M_MINOR = 31                   # 次轴等效弯矩系数 βm
    BETA_T_MAJOR = 32                   # 主轴弯矩系数 βt
    BETA_T_MINOR = 33                   # 次轴弯矩系数 βt
    PHI_MAJOR = 34                      # 主轴轴心稳定系数 φ
    PHI_MINOR = 35                      # 次轴轴心稳定系数 φ
    PHI_B_MAJOR = 36                    # 主轴弯曲稳定系数 φb
    PHI_B_MINOR = 37                    # 次轴弯曲稳定系数 φb
    GAMMA_MAJOR = 38                    # 主轴塑性发展系数 γ
    GAMMA_MINOR = 39                    # 次轴塑性发展系数 γ
    ETA_SECTION = 40                    # 截面影响系数 η
    ETA_BC = 41                         # 梁柱承载力系数 η
    DELTA_MAJOR = 42                    # 主轴欧拉弯矩系数 δ
    DELTA_MINOR = 43                    # 次轴欧拉弯矩系数 δ
    FY = 44                             # 屈服强度 Fy
    F_ALLOWABLE = 45                    # 容许正应力 f
    FV_ALLOWABLE = 46                   # 容许剪应力 fv
    CONSIDER_FICTITIOUS_SHEAR = 47      # 考虑虚拟剪力
    DC_RATIO_LIMIT = 48                 # 需求/承载力比限值
    DUAL_SYSTEM_FACTOR = 49             # 双重体系放大系数
    LOR_COMPRESSION = 50                # 受压长细比限值 Lo/r
    LR_TENSION = 51                     # 受拉长细比限值 L/r


class PreferenceItem(IntEnum):
    """首选项编号 (1-15)"""
    FRAMING_TYPE = 1                    # 结构体系类型
    GAMMA0 = 2                          # 重要性系数 γ0
    IGNORE_BT_CHECK = 3                 # 忽略宽厚比检查
    CLASSIFY_AS_FLEXO_COMPRESSION = 4   # 按压弯构件分类
    CONSIDER_DEFLECTION = 5             # 考虑挠度
    DL_DEFLECTION_RATIO = 6             # 恒载挠度限值 L/Value
    SDL_LL_DEFLECTION_RATIO = 7         # 附加恒载+活载挠度限值 L/Value
    LL_DEFLECTION_RATIO = 8             # 活载挠度限值 L/Value
    TOTAL_DEFLECTION_RATIO = 9          # 总荷载挠度限值 L/Value
    TOTAL_CAMBER_RATIO = 10             # 总起拱限值 L/Value
    PATTERN_LIVE_LOAD_FACTOR = 11       # 活载不利布置系数
    DC_RATIO_LIMIT = 12                 # 需求/承载力比限值
    MULTI_RESPONSE_DESIGN = 13          # 多工况设计方法
    IS_TALL_BUILDING = 14               # 是否高层建筑
    SEISMIC_DESIGN_GRADE = 15           # 抗震设计等级


@dataclass
class OverwriteResult:
    """覆盖项获取结果
    
    Attributes:
        value: 覆盖值
        prog_det: 是否程序确定
    """
    value: float
    prog_det: bool


# ============================================================================
# Preference 首选项函数
# ============================================================================

def get_chinese_2010_preference(model, item: Union[PreferenceItem, int]) -> float:
    """获取中国规范首选项值
    
    Args:
        model: SapModel 对象
        item: 首选项编号 (1-15)
        
    Returns:
        首选项值
    """
    result = model.DesignSteel.Chinese_2010.GetPreference(int(item), 0.0)
    if isinstance(result, (list, tuple)) and len(result) >= 2:
        return result[0]
    return 0.0


def set_chinese_2010_preference(model, item: Union[PreferenceItem, int], value: float) -> int:
    """设置中国规范首选项值
    
    Args:
        model: SapModel 对象
        item: 首选项编号 (1-15)
        value: 首选项值
        
    Returns:
        0 表示成功，非 0 表示失败
    """
    ret = model.DesignSteel.Chinese_2010.SetPreference(int(item), value)
    if isinstance(ret, (list, tuple)):
        return ret[-1]
    return ret


# ============================================================================
# Overwrite 覆盖项函数
# ============================================================================

def get_chinese_2010_overwrite(
    model,
    name: str,
    item: Union[OverwriteItem, int]
) -> OverwriteResult:
    """获取中国规范覆盖项值
    
    Args:
        model: SapModel 对象
        name: 框架对象名称
        item: 覆盖项编号 (1-51)
        
    Returns:
        覆盖项结果（值和是否程序确定）
    """
    result = model.DesignSteel.Chinese_2010.GetOverwrite(name, int(item), 0.0, False)
    if isinstance(result, (list, tuple)) and len(result) >= 3:
        return OverwriteResult(value=result[0], prog_det=bool(result[1]))
    return OverwriteResult(value=0.0, prog_det=True)


def set_chinese_2010_overwrite(
    model,
    name: str,
    item: Union[OverwriteItem, int],
    value: float,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """设置中国规范覆盖项值
    
    Args:
        model: SapModel 对象
        name: 对象名称、组名称或忽略（取决于 item_type）
        item: 覆盖项编号 (1-51)
        value: 覆盖值
        item_type: 对象选择类型
        
    Returns:
        0 表示成功，非 0 表示失败
    """
    ret = model.DesignSteel.Chinese_2010.SetOverwrite(name, int(item), value, int(item_type))
    if isinstance(ret, (list, tuple)):
        return ret[-1]
    return ret


# ============================================================================
# 便捷函数 - 常用首选项
# ============================================================================

def set_chinese_2010_framing_type(model, framing_type: FramingType) -> int:
    """设置结构体系类型"""
    return set_chinese_2010_preference(model, PreferenceItem.FRAMING_TYPE, float(framing_type))


def set_chinese_2010_gamma0(model, gamma0: float) -> int:
    """设置重要性系数 γ0"""
    return set_chinese_2010_preference(model, PreferenceItem.GAMMA0, gamma0)


def set_chinese_2010_seismic_grade(model, grade: SeismicDesignGrade) -> int:
    """设置抗震设计等级"""
    return set_chinese_2010_preference(model, PreferenceItem.SEISMIC_DESIGN_GRADE, float(grade))


def set_chinese_2010_dc_ratio_limit(model, ratio: float) -> int:
    """设置需求/承载力比限值"""
    return set_chinese_2010_preference(model, PreferenceItem.DC_RATIO_LIMIT, ratio)


def set_chinese_2010_tall_building(model, is_tall: bool) -> int:
    """设置是否高层建筑"""
    return set_chinese_2010_preference(model, PreferenceItem.IS_TALL_BUILDING, 1.0 if is_tall else 0.0)


# ============================================================================
# 便捷函数 - 常用覆盖项
# ============================================================================

def set_chinese_2010_element_type(
    model,
    name: str,
    element_type: ElementType,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """设置构件类型"""
    return set_chinese_2010_overwrite(model, name, OverwriteItem.ELEMENT_TYPE, float(element_type), item_type)


def set_chinese_2010_mue_factors(
    model,
    name: str,
    mue_major: float,
    mue_minor: float,
    item_type: ItemType = ItemType.OBJECT
) -> Tuple[int, int]:
    """设置计算长度系数 μ
    
    Args:
        model: SapModel 对象
        name: 对象名称
        mue_major: 主轴计算长度系数
        mue_minor: 次轴计算长度系数
        item_type: 对象选择类型
        
    Returns:
        (主轴返回码, 次轴返回码)
    """
    ret1 = set_chinese_2010_overwrite(model, name, OverwriteItem.MUE_MAJOR, mue_major, item_type)
    ret2 = set_chinese_2010_overwrite(model, name, OverwriteItem.MUE_MINOR, mue_minor, item_type)
    return ret1, ret2


def set_chinese_2010_unbraced_ratios(
    model,
    name: str,
    ratio_major: float,
    ratio_minor: float,
    item_type: ItemType = ItemType.OBJECT
) -> Tuple[int, int]:
    """设置无支撑长度比
    
    Args:
        model: SapModel 对象
        name: 对象名称
        ratio_major: 主轴无支撑长度比
        ratio_minor: 次轴无支撑长度比（侧扭屈曲）
        item_type: 对象选择类型
        
    Returns:
        (主轴返回码, 次轴返回码)
    """
    ret1 = set_chinese_2010_overwrite(model, name, OverwriteItem.UNBRACED_RATIO_MAJOR, ratio_major, item_type)
    ret2 = set_chinese_2010_overwrite(model, name, OverwriteItem.UNBRACED_RATIO_MINOR_LTB, ratio_minor, item_type)
    return ret1, ret2
